unit STUIniFile_U;
// Description: SecureTrayUtil INI File
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//
// Defines INI file contents and methods for reading and writing them


interface

uses inifiles,
     dialogs,
     forms,
     classes,
     sysutils,
     Windows, // needed for WritePrivateProfileString
     STUIniFileConsts_U,
     AppGlobalConsts_U,
     OTFEUnified_U,
     OTFETrueCryptStructures_U,
     HashAlgUnified_U;

type
  TOnExit = (ExitNone, ExitDismount, ExitDismountBrutal);
  TOnDblClk = (DblClkSlotInfo,
               DblClkDismount, DblClkDismountBrutal,
               DblClkOpenPromptDismount, DblClkOpenPromptDismountBrutal,
               DblClkOpenPromptDismountPrompt,
               DblClkOpenPrompt, DblClkDismountPrompt);

  TSelfDMenuitem = (sdmiTop, sdmiBottom, sdmiBoth);
  TSelfDMethod = (sdmhDoNot, sdmhInternal, sdmhRunExe);

  TTouchFileTarget = class
  public
    filename: string;
    datetime: TDateTime;
    useCurrentDateTime: boolean;
  end;

  TSTUIniFile = class(TINIFile)
  private
    function ReadIniFileGeneral() : boolean;
    function ReadIniFileVolInfoDlg() : boolean;
    function ReadIniFileSelfD() : boolean;
    function ReadIniFileShellExt() : boolean;
    function ReadIniFileHash() : boolean;
    function ReadIniFileOTFCryptoSupport() : boolean;
    function ReadTouchFiles(): boolean;

    function WriteIniFileGeneral() : boolean;
    function WriteIniFileVolInfoDlg() : boolean;
    function WriteIniFileSelfD() : boolean;
    function WriteIniFileShellExt() : boolean;
    function WriteIniFileHash() : boolean;
    function WriteIniFileOTFCryptoSupport() : boolean;
    function WriteTouchFiles(): boolean;
  public
    NewVersion: boolean; // Set if this if there is no INI file or if it's for a previous version
    
    // General options
    OptShowVolInfoOnStartup: boolean;
    OptUseTaskTrayIcon: boolean;
    OptConfirmOnDismount: boolean;

    // Tasktray icon selection
    OptTaskTrayIconUnmounted: integer;
    OptTaskTrayIconMounted: integer;
    OptTaskTrayIconUnsure: integer;
    OptTaskTrayAlternateMenu: boolean;

    // General options
    OptHKeyVolInfoEnable: boolean;
    OptHKeyVolInfoKey: string;
    OptHKeyScreenSaverEnable: boolean;
    OptHKeyScreenSaverKey: string;
    OptHKeyExitEnable: boolean;
    OptHKeyExitKey: string;

    // General Shredder Methods
    OptShredFileDirUseInt: boolean;
    OptShredFreeUseInt: boolean;
    OptShredSlack: boolean;
    OptShredExtFileExe : string;
    OptShredExtDirExe : string;
    OptShredExtFreeSpaceExe: string;
    OptShredExtDirDiff : boolean; // TRUE if should use different command line for shredding dirs
    OptShredExtShredFilesThenDir : boolean; // TRUE if should shred all files in a dir before shredding the dir
    OptShredIntFirstBytes : longint;
    OptShredIntPasses : integer;
    OptShredIntGutmann: boolean;
    OptShredIntShowProgress: boolean;
    OptShredIntFreeSpcFileSize: integer;
    OptShredIntFreeSpcFileCreationBlkSize: integer;
    OptShredIntFileBufferSize: integer;

    // Volume Info dialog setup
    OptDlgSize_VolInfo_IconView : boolean;
    OptDlgSize_VolInfo_TreeView : boolean;
    OptDlgSize_VolInfo_TableView: boolean;
    OptDlgSize_VolInfo_DispNonEncrypted: boolean;
    OptDlgSize_VolInfo_DispVolInfoPnl: boolean;
    OptDlgSize_VolInfo_Height     : integer;
    OptDlgSize_VolInfo_Width      : integer;
    OptDlgSize_VolInfo_Top        : integer;
    OptDlgSize_VolInfo_Left       : integer;
    OptDlgSize_VolInfo_WindowState: TWindowState;
    OptDlgSize_VolInfo_ColPosDrive      : integer;
    OptDlgSize_VolInfo_ColPosVolLabel   : integer;
    OptDlgSize_VolInfo_ColPosVolFilename: integer;
    OptDlgSize_VolInfo_ColWidthDrive      : integer;
    OptDlgSize_VolInfo_ColWidthVolLabel   : integer;
    OptDlgSize_VolInfo_ColWidthVolFilename: integer;

    // OTF Crypto systems
    OptOTFCryptoSupport: array [TOTFESystem] of boolean;
    OptOTFCryptoDefault: TOTFESystem;
    OptOTFDefaultBrowseDir: string;
    OptOTFDefaultBrowseFilter: integer;
    OptOTFExploreAfterMount: boolean;
    OptOTFAlwaysUnmountedIcon: boolean;
    OptOTFMountVolStart : boolean;
    OptOTFMountPartStart : boolean;
    OptOTFOnExit : TOnExit;
    OptOTFOnDblClk : TOnDblClk;
    OptOTFHidePasswordsEnteredCk: boolean;
    // OTF Crypto system pre/post mount executables
    OptOTFPostMountExe: string;
    OptOTFPreDismountExe: string;
    OptOTFPostDismountExe: string;
    OptOTFPrePostWarnNotFound: boolean;
    // OTF Crypto system hotkeys
    OptOTFHKeyDismountEnable : boolean;
    OptOTFHKeyDismountBrutalEnable : boolean;
    OptOTFHKeyDismountKey : string;
    OptOTFHKeyDismountBrutalKey : string;

    // OTF Crypto systems: CrossCrypt
    OptOTFCrossCryptExe: string;

    // OTF Crypto systems: E4M
    OptOTFE4MExe: string;
    OptOTFE4MMountDeviceDlg9xStyle: boolean;
    OptOTFE4MSupportScramDisk: boolean;

    // OTF Crypto systems: FreeOTFE
    OptOTFFreeOTFEExe: string;

    // OTF Crypto systems: TrueCrypt
    OptOTFTrueCryptVersionHint: TTrueCryptVersionHint;
    OptOTFTrueCryptMountDeviceDlg9xStyle: boolean;

    // ScramDisk General options
    OptOTFScramDiskExe: string;
    OptOTFScramDiskUseRedScreen: boolean;
    OptOTFScramDiskDisableSVLProtection: boolean;
    OptOTFScramDiskPrefDrive: string;

    // Touch files
    OptTouchTargetFiles: TStringList;
    OptTouchFrequency: cardinal;
    OptTouchRegularly: boolean;
    OptTouchOnStart: boolean;
    OptTouchOnExit: boolean;

    // SecureTrayUtil Shell Extensions
    OptShellExtSubMenu        : boolean;
    OptShellExtSubMenuTitle   : string;
    OptShellExtMenuSeparators : boolean;
    OptShellExtMenuMountVolume: boolean;
    OptShellExtMenuShredFile  : boolean;
    OptShellExtMenuShredDir   : boolean;
    OptShellExtMenuShredFreeSpace: boolean;
    OptShellExtMenuTouch      : boolean;
    OptShellExtMenuHash       : boolean;
    OptShellExtAppPath        : boolean;

    // Hash
    OptHashEnabled: array [fhHashType] of boolean;
    OptHashInUppercase: boolean;

    // Self-destruct Options
    OptSelfDFacilitiesEnable1 : boolean;
    OptSelfDFacilitiesEnable2 : boolean;

    // Self-destruct Triggers
    OptSelfDEnableHotkey : boolean;
    OptSelfDHotkey : string;
    OptSelfDTriggerPortEnable: boolean;
    OptSelfDTriggerPort: string;
    OptSelfDTriggerPortInterval: integer;
    OptSelfDTriggerTestMode: boolean;
    OptSelfDEnableInternetTrigger: boolean;
    OptSelfDEnableMenuitem : boolean;
    OptSelfDMenuitemLocation : TSelfDMenuitem;
    OptSelfDInternetPort: integer;
    OptSelfDInternetCRPairs: integer;
    OptSelfDInternetChal: array of string;
    OptSelfDInternetResp: array of string;

    // Self-destruct Targets
    OptSelfDDestroyMountedEnable : boolean;
    OptSelfDDestroyMountedQuickShred: boolean;
    OptSelfDTargetFiles: TStringList;

    // Self-destruct Additional
    OptSelfDRunAdditionalEnable : boolean;
    OptSelfDRunAdditionalFilename : string;
    OptSelfDStealth: boolean;
    OptSelfDRandomizeDT: boolean;
    OptSelfDRandomizeDTStart: TDateTime;
    OptSelfDRandomizeDTEnd: TDateTime;
    OptSelfDRandomizeDTStartCurrent: boolean;
    OptSelfDRandomizeDTEndCurrent: boolean;

    constructor Create(const FileName : string);
    destructor  Destroy(); override;

    function ReadIniFile() : boolean;
    function WriteIniFile() : boolean;

    procedure ClearTouchTargetFilesList();
    procedure CheckVersionID();
  end;

implementation

uses
     AppGlobal_U,
     SDUGeneral;

constructor TSTUIniFile.Create(const FileName : string);
begin
  inherited;
  OptTouchTargetFiles := TStringList.create();
  OptTouchTargetFiles.clear;

  OptSelfDTargetFiles := TStringList.create();
  OptSelfDTargetFiles.clear;

end;

destructor TSTUIniFile.Destroy();
begin
  ClearTouchTargetFilesList();
  OptTouchTargetFiles.Free();

  OptSelfDTargetFiles.Free();

  inherited;
end;

function TSTUIniFile.ReadIniFile() : boolean;
var
  retVal : boolean;
begin
  retVal := ReadIniFileGeneral();
  retVal := retVal AND ReadIniFileVolInfoDlg();
  retVal := retVal AND ReadTouchFiles();
  retVal := retVal AND ReadIniFileSelfD();
  retVal := retVal AND ReadIniFileShellExt();
  retVal := retVal AND ReadIniFileHash();
  retVal := retVal AND ReadIniFileOTFCryptoSupport();

  Result := retVal;
end;

function TSTUIniFile.ReadIniFileGeneral() : boolean;
var
  tempInt: integer;
begin
  OptShowVolInfoOnStartup := ReadBool(SECTION_SECURETRAYUTIL, KEY_STARTUPSHOWVOLINFODLG, DFLT_KEY_STARTUPSHOWVOLINFODLG);
  OptUseTaskTrayIcon := ReadBool(SECTION_SECURETRAYUTIL, KEY_USEDISPLAYTASKTRAYICON, DFLT_KEY_USEDISPLAYTASKTRAYICON);
  OptConfirmOnDismount := ReadBool(SECTION_SECURETRAYUTIL, KEY_CONFIRMONDISMOUNT, DFLT_KEY_CONFIRMONDISMOUNT);

  OptTaskTrayIconUnmounted := ReadInteger(SECTION_SECURETRAYUTIL, KEY_TASKTRAY_ICON_UNMOUNTED,  DFLT_KEY_TASKTRAY_ICON_UNMOUNTED);
  OptTaskTrayIconMounted   := ReadInteger(SECTION_SECURETRAYUTIL, KEY_TASKTRAY_ICON_MOUNTED,    DFLT_KEY_TASKTRAY_ICON_MOUNTED);
  OptTaskTrayIconUnsure    := ReadInteger(SECTION_SECURETRAYUTIL, KEY_TASKTRAY_ICON_UNSURE,     DFLT_KEY_TASKTRAY_ICON_UNSURE);
  OptTaskTrayAlternateMenu := ReadBool(SECTION_SECURETRAYUTIL,    KEY_USEALTERNATETASKTRAYMENU, DFLT_KEY_USEALTERNATETASKTRAYMENU);

  OptHKeyVolInfoEnable := ReadBool(SECTION_SECURETRAYUTIL, KEY_VOLINFOHOTKEYENABLE, DFLT_KEY_VOLINFOHOTKEYENABLE);
  OptHKeyVolInfoKey := ReadString(SECTION_SECURETRAYUTIL, KEY_VOLINFOHOTKEY, DFLT_KEY_VOLINFOHOTKEY);
  OptHKeyScreenSaverEnable := ReadBool(SECTION_SECURETRAYUTIL, KEY_SCREENSAVERHOTKEYENABLE, DFLT_KEY_SCREENSAVERHOTKEYENABLE);
  OptHKeyScreenSaverKey := ReadString(SECTION_SECURETRAYUTIL, KEY_SCREENSAVERHOTKEY, DFLT_KEY_SCREENSAVERHOTKEY);
  OptHKeyExitEnable := ReadBool(SECTION_SECURETRAYUTIL, KEY_EXITHOTKEYENABLE, DFLT_KEY_EXITHOTKEYENABLE);
  OptHKeyExitKey := ReadString(SECTION_SECURETRAYUTIL, KEY_EXITHOTKEY, DFLT_KEY_EXITHOTKEY);

  OptShredFileDirUseInt := ReadBool(SECTION_SECURETRAYUTIL, KEY_SHREDFILEDIRUSEINT, DFLT_KEY_SHREDFILEDIRUSEINT);
  OptShredFreeUseInt := ReadBool(SECTION_SECURETRAYUTIL, KEY_SHREDFREEUSEINT, DFLT_KEY_SHREDFREEUSEINT);
  OptShredSlack := ReadBool(SECTION_SECURETRAYUTIL, KEY_SHREDSLACK, DFLT_KEY_SHREDSLACK);

  OptShredExtFileExe := ReadString(SECTION_SECURETRAYUTIL, KEY_SHREDEXTFILEEXE, DFLT_KEY_SHREDEXTFILEEXE);
  OptShredExtDirExe := ReadString(SECTION_SECURETRAYUTIL, KEY_SHREDEXTDIREXE, DFLT_KEY_SHREDEXTDIREXE);
  OptShredExtFreeSpaceExe := ReadString(SECTION_SECURETRAYUTIL, KEY_SHREDEXTFREESPACEEXE, DFLT_KEY_SHREDEXTFREESPACEEXE);
  OptShredExtDirDiff := ReadBool(SECTION_SCRAMDISK, KEY_SHREDEXTDIRDIFF, DFLT_KEY_SHREDEXTDIRDIFF);
  OptShredExtShredFilesThenDir := ReadBool(SECTION_SCRAMDISK, KEY_SHREDEXTFILESTHENDIR, DFLT_KEY_SHREDEXTFILESTHENDIR);
  OptShredIntFirstBytes := ReadInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFIRSTBYTES, DFLT_KEY_SHREDINTFIRSTBYTES);
  OptShredIntPasses := ReadInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTPASSES, DFLT_KEY_SHREDINTPASSES);
  OptShredIntGutmann := ReadBool(SECTION_SECURETRAYUTIL, KEY_SHREDINTUSEGUTMANN, DFLT_KEY_SHREDINTUSEGUTMANN);
  OptShredIntShowProgress :=  ReadBool(SECTION_SECURETRAYUTIL, KEY_SHREDINTSHOWPROGRESS, DFLT_KEY_SHREDINTSHOWPROGRESS);

  OptShredIntFreeSpcFileSize := ReadInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFREESPCFILESZ, DFLT_KEY_SHREDINTFREESPCFILESZ);
  OptShredIntFreeSpcFileCreationBlkSize := ReadInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFREESPCFILECREATIONBLKSZ, DFLT_KEY_SHREDINTFREESPCFILECREATIONBLKSZ);
  OptShredIntFileBufferSize := ReadInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFILEBUFSZ, DFLT_KEY_SHREDINTFILEBUFSZ);

  OptOTFScramDiskExe := ReadString(SECTION_SCRAMDISK, KEY_SCRAMDISKEXE, DFLT_KEY_SCRAMDISKEXE);
  OptOTFScramDiskUseRedScreen := ReadBool(SECTION_SCRAMDISK, KEY_USEREDSCREEN, DFLT_KEY_USEREDSCREEN);
  OptOTFScramDiskDisableSVLProtection := ReadBool(SECTION_SCRAMDISK, KEY_DISABLESVLPROTECTION, DFLT_KEY_DISABLESVLPROTECTION);
  OptOTFScramDiskPrefDrive := ReadString(SECTION_SCRAMDISK, KEY_SCRAMDISKPREFDRIVE, DFLT_KEY_SCRAMDISKPREFDRIVE);

  OptOTFCrossCryptExe := ReadString(SECTION_OTFCROSSCRYPT, KEY_CROSSCRYPTEXE, DFLT_KEY_CROSSCRYPTEXE);

  OptOTFE4MExe := ReadString(SECTION_OTFE4M, KEY_E4MEXE, DFLT_KEY_E4MEXE);
  OptOTFE4MMountDeviceDlg9xStyle := ReadBool(SECTION_OTFE4M, KEY_E4MMOUNTDEVICEDLG9XSTYLE, DFLT_KEY_E4MMOUNTDEVICEDLG9XSTYLE);
  OptOTFE4MSupportScramDisk := ReadBool(SECTION_OTFE4M, KEY_SUPPORTSCRAMDISK, DFLT_KEY_SUPPORTSCRAMDISK);

  OptOTFFreeOTFEExe := ReadString(SECTION_OTFFREEOTFE, KEY_FREEOTFEEXE, DFLT_KEY_FREEOTFEEXE);

  // BEFORE CHANGING THIS!!!
  // These numbers need to be kept the same - otherwise the user's options will
  // mysteriously change
  tempInt := ReadInteger(SECTION_OTFTRUECRYPT, KEY_VERSIONHINT, DFLT_KEY_VERSIONHINT);
  case tempInt of
    1 : OptOTFTrueCryptVersionHint := tcvAuto;
    2 : OptOTFTrueCryptVersionHint := tcv21;
    3 : OptOTFTrueCryptVersionHint := tcv21a;
  else
    OptOTFTrueCryptVersionHint := tcvAuto; // Default and in case of error
  end;
  OptOTFTrueCryptMountDeviceDlg9xStyle := ReadBool(SECTION_OTFTRUECRYPT, KEY_TRUECRYPTMOUNTDEVICEDLG9XSTYLE, DFLT_KEY_TRUECRYPTMOUNTDEVICEDLG9XSTYLE);

  Result := TRUE;
end;

function TSTUIniFile.ReadIniFileVolInfoDlg() : boolean;
var
  tempInt: integer;
begin
  OptDlgSize_VolInfo_IconView  := ReadBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_ICONVIEW, DFLT_KEY_VOLINFO_DIALOG_ICONVIEW);
  OptDlgSize_VolInfo_TreeView  := ReadBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_TREEVIEW, DFLT_KEY_VOLINFO_DIALOG_TREEVIEW);
  OptDlgSize_VolInfo_TableView := ReadBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_TABLEVIEW, DFLT_KEY_VOLINFO_DIALOG_TABLEVIEW);

  OptDlgSize_VolInfo_DispNonEncrypted := ReadBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_DISPNONENCRYPTED, DFLT_KEY_VOLINFO_DIALOG_DISPNONENCRYPTED);
  OptDlgSize_VolInfo_DispVolInfoPnl := ReadBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_DISPVOLINFOPANEL, DFLT_KEY_VOLINFO_DIALOG_DISPVOLINFOPANEL);

  OptDlgSize_VolInfo_Height     := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_HEIGHT, DFLT_KEY_VOLINFO_DIALOG_HEIGHT);
  OptDlgSize_VolInfo_Width      := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_WIDTH,  DFLT_KEY_VOLINFO_DIALOG_WIDTH);
  OptDlgSize_VolInfo_Top        := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_TOP,    DFLT_KEY_VOLINFO_DIALOG_TOP);
  OptDlgSize_VolInfo_Left       := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_LEFT,   DFLT_KEY_VOLINFO_DIALOG_LEFT);

  tempInt := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_WINDOWSTATE, -1);
  case tempInt of
    1 : OptDlgSize_VolInfo_WindowState := wsMinimized;
    2 : OptDlgSize_VolInfo_WindowState := wsMaximized;
  else
    OptDlgSize_VolInfo_WindowState := wsNormal; // Default and in case of error
  end;

  OptDlgSize_VolInfo_ColPosDrive      := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLPOSDRIVE,       DFLT_KEY_VOLINFO_DIALOG_COLPOSDRIVE);
  OptDlgSize_VolInfo_ColPosVolLabel   := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLPOSVOLLABEL,    DFLT_KEY_VOLINFO_DIALOG_COLPOSVOLLABEL);
  OptDlgSize_VolInfo_ColPosVolFilename:= ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLPOSVOLFILENAME, DFLT_KEY_VOLINFO_DIALOG_COLPOSVOLFILENAME);

  OptDlgSize_VolInfo_ColWidthDrive      := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLWIDTHDRIVE,       DFLT_KEY_VOLINFO_DIALOG_COLWIDTHDRIVE);
  OptDlgSize_VolInfo_ColWidthVolLabel   := ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLWIDTHVOLLABEL,    DFLT_KEY_VOLINFO_DIALOG_COLWIDTHVOLLABEL);
  OptDlgSize_VolInfo_ColWidthVolFilename:= ReadInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLWIDTHVOLFILENAME, DFLT_KEY_VOLINFO_DIALOG_COLWIDTHVOLFILENAME);

  Result := TRUE;

end;

function TSTUIniFile.WriteIniFileVolInfoDlg() : boolean;
var
  tempInt: integer;
begin
  case OptDlgSize_VolInfo_WindowState of
    wsMinimized: tempInt := 1;
    wsMaximized: tempInt := 2;
  else
    tempInt := 0; // Default and in case of error
  end;
  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_WINDOWSTATE, tempInt);

  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLPOSDRIVE,       OptDlgSize_VolInfo_ColPosDrive);
  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLPOSVOLLABEL,    OptDlgSize_VolInfo_ColPosVolLabel);
  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLPOSVOLFILENAME, OptDlgSize_VolInfo_ColPosVolFilename);

  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLWIDTHDRIVE,       OptDlgSize_VolInfo_ColWidthDrive);
  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLWIDTHVOLLABEL,    OptDlgSize_VolInfo_ColWidthVolLabel);
  WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_COLWIDTHVOLFILENAME, OptDlgSize_VolInfo_ColWidthVolFilename);

  WriteBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_ICONVIEW, OptDlgSize_VolInfo_IconView);
  WriteBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_TREEVIEW, OptDlgSize_VolInfo_TreeView);
  WriteBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_TABLEVIEW, OptDlgSize_VolInfo_TableView);

  WriteBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_DISPNONENCRYPTED, OptDlgSize_VolInfo_DispNonEncrypted);
  WriteBool(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_DISPVOLINFOPANEL, OptDlgSize_VolInfo_DispVolInfoPnl);

  if OptDlgSize_VolInfo_WindowState<>wsMaximized then
    begin
    WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_HEIGHT, OptDlgSize_VolInfo_Height);
    WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_WIDTH,  OptDlgSize_VolInfo_Width);
    WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_TOP,    OptDlgSize_VolInfo_Top);
    WriteInteger(SECTION_VOLINFO_DLG, KEY_VOLINFO_DIALOG_LEFT,   OptDlgSize_VolInfo_Left);
    end;

  Result := TRUE;

end;

function TSTUIniFile.ReadIniFileSelfD() : boolean;
var
  tempInt : integer;
  tempString : string;
  count : integer;
  i : integer;
  keyFilename : string;
begin
  OptSelfDFacilitiesEnable1 := ReadBool(SECTION_SELFD, KEY_FACILITIESENABLE1, DFLT_KEY_FACILITIESENABLE1);
  OptSelfDFacilitiesEnable2 := ReadBool(SECTION_SELFD, KEY_FACILITIESENABLE2, DFLT_KEY_FACILITIESENABLE2);

  // BEFORE CHANGING THIS!!!
  // These numbers need to be kept the same - otherwise the user's options will
  // mysteriously change
  tempInt := ReadInteger(SECTION_SELFD, KEY_MENUTRIGGERLOCATION, DFLT_KEY_MENUTRIGGERLOCATION);
  case tempInt of
    1 : OptSelfDMenuitemLocation := sdmiTop;
    2 : OptSelfDMenuitemLocation := sdmiBottom;
  else
    OptSelfDMenuitemLocation := sdmiBoth; // Default and in case of error
  end;
  OptSelfDEnableMenuitem := ReadBool(SECTION_SELFD, KEY_ENABLEMENUTRIGGER, DFLT_KEY_ENABLEMENUTRIGGER);

  OptSelfDEnableHotkey := ReadBool(SECTION_SELFD, KEY_ENABLESELFDHOTKEY, DFLT_KEY_ENABLESELFDHOTKEY);
  OptSelfDHotkey := ReadString(SECTION_SELFD, KEY_SELFDHOTKEY, DFLT_KEY_SELFDHOTKEY);
  OptSelfDTriggerPortEnable := ReadBool(SECTION_SELFD, KEY_ENABLESERIALTRIGGER, DFLT_KEY_ENABLESERIALTRIGGER);
  OptSelfDTriggerPort := ReadString(SECTION_SELFD, KEY_SERIALTRIGGERPORT, DFLT_KEY_SERIALTRIGGERPORT);
  OptSelfDTriggerPortInterval := ReadInteger(SECTION_SELFD, KEY_SERIALTRIGGERINTERVAL, DFLT_KEY_SERIALTRIGGERINTERVAL);
  OptSelfDEnableInternetTrigger := ReadBool(SECTION_SELFD, KEY_ENABLEINTERNETTRIGGER, DFLT_KEY_ENABLEINTERNETTRIGGER);
  OptSelfDInternetPort := ReadInteger(SECTION_SELFD, KEY_INTERNETTRIGGERPORT, DFLT_KEY_INTERNETTRIGGERPORT);
  OptSelfDInternetCRPairs := ReadInteger(SECTION_SELFD, KEY_INTERNETTRIGGERCRPAIRS, DFLT_KEY_INTERNETTRIGGERCRPAIRS);

  SetLength(OptSelfDInternetChal, OptSelfDInternetCRPairs);
  SetLength(OptSelfDInternetResp, OptSelfDInternetCRPairs);
  for i:=1 to OptSelfDInternetCRPairs do
    begin
    OptSelfDInternetChal[i-1] := ReadString(SECTION_SELFD, KEY_INTERNETTRIGGERCHAL+inttostr(i), DFLT_KEY_INTERNETTRIGGERCHAL);
    OptSelfDInternetResp[i-1] := ReadString(SECTION_SELFD, KEY_INTERNETTRIGGERRESP+inttostr(i), DFLT_KEY_INTERNETTRIGGERRESP);
    end;

  OptSelfDTriggerTestMode := ReadBool(SECTION_SELFD, KEY_TRIGGERTESTMODE, DFLT_KEY_TRIGGERTESTMODE);

  OptSelfDDestroyMountedEnable := ReadBool(SECTION_SELFD, KEY_DESTROYMOUNTEDENABLE, DFLT_KEY_DESTROYMOUNTEDENABLE);
  OptSelfDDestroyMountedQuickShred := ReadBool(SECTION_SELFD, KEY_DESTROYMOUNTEDQUICK, DFLT_KEY_DESTROYMOUNTEDQUICK);
  OptSelfDRunAdditionalEnable := ReadBool(SECTION_SELFD, KEY_RUNADDITIONALENABLE, DFLT_KEY_RUNADDITIONALENABLE);
  OptSelfDRunAdditionalFilename := ReadString(SECTION_SELFD, KEY_RUNADDITIONALFILENAME, DFLT_KEY_RUNADDITIONALFILENAME);
  OptSelfDStealth := ReadBool(SECTION_SELFD, KEY_STEALTHSELFD, DFLT_KEY_STEALTHSELFD);

  OptSelfDRandomizeDT := ReadBool(SECTION_SELFD, KEY_RANDOMIZECLOCK, DFLT_KEY_RANDOMIZECLOCK);
  OptSelfDRandomizeDTStart := StrToDateTime(ReadString(SECTION_SELFD, KEY_RANDOMIZECLOCKSTARTDATETIME, DateTimeToStr(Now)));
  OptSelfDRandomizeDTEnd := StrToDateTime(ReadString(SECTION_SELFD, KEY_RANDOMIZECLOCKENDDATETIME, DateTimeToStr(Now)));
  OptSelfDRandomizeDTStartCurrent := ReadBool(SECTION_SELFD, KEY_RANDOMIZECLOCKSTARTCURRENT, DFLT_KEY_RANDOMIZECLOCKSTARTCURRENT);
  OptSelfDRandomizeDTEndCurrent := ReadBool(SECTION_SELFD, KEY_RANDOMIZECLOCKENDCURRENT, DFLT_KEY_RANDOMIZECLOCKENDCURRENT);

  // Last, read in the target files
  count := ReadInteger(SECTION_SELFDTARGETS, KEY_TARGETSCOUNT, DFLT_KEY_TARGETSCOUNT);
  OptSelfDTargetFiles.clear;
  for i:=1 to count do
    begin
    keyFilename := KEY_TARGET + inttostr(i-1); // -1 for backward compatability
    tempString := ReadString(SECTION_SELFDTARGETS, keyFilename, DFLT_KEY_TARGET);
    OptSelfDTargetFiles.add(tempString);
    end;

  Result := TRUE;
end;

function TSTUIniFile.ReadIniFileShellExt() : boolean;
begin
  OptShellExtSubMenu         := ReadBool(SECTION_SHELLEXT, KEY_SUBMENU, DFLT_KEY_SUBMENU);
  OptShellExtSubMenuTitle    := ReadString(SECTION_SHELLEXT, KEY_SUBMENUTITLE, DFLT_KEY_SUBMENUTITLE);
  OptShellExtMenuSeparators  := ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_SEPARATORS, DFLT_KEY_MENUITEM_SEPARATORS);

  OptShellExtMenuMountVolume := ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_MOUNTVOLUME, DFLT_KEY_MENUITEM_MOUNTVOLUME);
  OptShellExtMenuShredFile   := ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_SHREDFILE, DFLT_KEY_MENUITEM_SHREDFILE);
  OptShellExtMenuShredDir    := ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_SHREDDIR, DFLT_KEY_MENUITEM_SHREDDIR);
  OptShellExtMenuShredFreeSpace:= ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_SHREDFREESPACE, DFLT_KEY_MENUITEM_SHREDFREESPACE);
  OptShellExtMenuTouch       := ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_TOUCH, DFLT_KEY_MENUITEM_TOUCH);
  OptShellExtMenuHash        := ReadBool(SECTION_SHELLEXT, KEY_MENUITEM_HASH, DFLT_KEY_MENUITEM_HASH);
  OptShellExtAppPath         := ReadBool(SECTION_SHELLEXT, KEY_APPPATH, DFLT_KEY_APPPATH);
  Result := TRUE;
end;

function TSTUIniFile.ReadIniFileHash() : boolean;
var
  hashLoop: fhHashType;
  currKey: string;
  hashUnified: THashAlgUnified;
begin
  hashUnified:= THashAlgUnified.Create(nil);
  try
    for hashLoop:=low(hashLoop) to high(hashLoop) do
      begin
      hashUnified.ActiveHash := hashLoop;
      currKey := KEY_HASH_ENABLEHASH + hashUnified.Title;
      OptHashEnabled[hashLoop] := ReadBool(SECTION_HASH, currKey, DFLT_KEY_HASH_ENABLEHASH);
      end;

  finally
    hashUnified.Free();
  end;

  OptHashInUppercase := ReadBool(SECTION_HASH, KEY_HASH_UPPERCASE, DFLT_KEY_HASH_UPPERCASE);

  Result := TRUE;
end;

function TSTUIniFile.WriteIniFile() : boolean;
var
  retVal : boolean;
begin
  retVal := WriteIniFileGeneral();
  retVal := retVal AND WriteIniFileVolInfoDlg();
  retVal := retVal AND WriteTouchFiles();
  retVal := retVal AND WriteIniFileSelfD();
  retVal := retVal AND WriteIniFileShellExt();
  retVal := retVal AND WriteIniFileHash();
  retVal := retVal AND WriteIniFileOTFCryptoSupport();

  // In order to ensure that the INI file is flushed and the changes are made
  // permanent, call the Windows API function WriteProfileString() or
  // WritePrivateProfileString() and pass nil parameters for the
  // section, key, and string values. [from Borland FAQ]
  UpdateFile(); // Calls WritePrivateProfileString()

  Result := retVal;
end;

function TSTUIniFile.WriteIniFileGeneral() : boolean;
var
  thisMajorVersion   : integer;
  thisMinorVersion   : integer;
  thisRevisionVersion: integer;
  thisBuildVersion   : integer;

  strVersion: string;
  tempInt: integer;
begin
  SDUGetVersionInfo('', thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion);
  strVersion := SDUGetVersionInfoString('');
  if (APP_BETA_BUILD>-1) then
    begin
    strVersion := strVersion + ' BETA ' + inttostr(APP_BETA_BUILD);
    end;

  WriteString(SECTION_SECURETRAYUTIL, KEY_VERSION, strVersion);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_MAJOR_VERSION,   thisMajorVersion);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_MINOR_VERSION,   thisMinorVersion);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_RELEASE_VERSION, thisRevisionVersion);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_BUILD_VERSION,   thisBuildVersion);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_BETA_VERSION,    APP_BETA_BUILD);

  WriteBool(SECTION_SECURETRAYUTIL, KEY_STARTUPSHOWVOLINFODLG, OptShowVolInfoOnStartup);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_USEDISPLAYTASKTRAYICON, OptUseTaskTrayIcon);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_CONFIRMONDISMOUNT, OptConfirmOnDismount);

  WriteInteger(SECTION_SECURETRAYUTIL, KEY_TASKTRAY_ICON_UNMOUNTED, OptTaskTrayIconUnmounted);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_TASKTRAY_ICON_MOUNTED,   OptTaskTrayIconMounted);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_TASKTRAY_ICON_UNSURE,    OptTaskTrayIconUnsure);
  WriteBool(SECTION_SECURETRAYUTIL,    KEY_USEALTERNATETASKTRAYMENU, OptTaskTrayAlternateMenu);

  WriteBool(SECTION_SECURETRAYUTIL, KEY_VOLINFOHOTKEYENABLE, OptHKeyVolInfoEnable);
  WriteString(SECTION_SECURETRAYUTIL, KEY_VOLINFOHOTKEY, OptHKeyVolInfoKey);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_SCREENSAVERHOTKEYENABLE, OptHKeyScreenSaverEnable);
  WriteString(SECTION_SECURETRAYUTIL, KEY_SCREENSAVERHOTKEY, OptHKeyScreenSaverKey);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_EXITHOTKEYENABLE, OptHKeyExitEnable);
  WriteString(SECTION_SECURETRAYUTIL, KEY_EXITHOTKEY, OptHKeyExitKey);

  WriteBool(SECTION_SECURETRAYUTIL, KEY_SHREDFILEDIRUSEINT, OptShredFileDirUseInt);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_SHREDFREEUSEINT, OptShredFreeUseInt);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_SHREDSLACK, OptShredSlack);

  WriteString(SECTION_SECURETRAYUTIL, KEY_SHREDEXTFILEEXE, OptShredExtFileExe);
  WriteString(SECTION_SECURETRAYUTIL, KEY_SHREDEXTDIREXE, OptShredExtDirExe);
  WriteString(SECTION_SECURETRAYUTIL, KEY_SHREDEXTFREESPACEEXE, OptShredExtFreeSpaceExe);
  WriteBool(SECTION_SCRAMDISK, KEY_SHREDEXTDIRDIFF, OptShredExtDirDiff);
  WriteBool(SECTION_SCRAMDISK, KEY_SHREDEXTFILESTHENDIR, OptShredExtShredFilesThenDir);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFIRSTBYTES, OptShredIntFirstBytes);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTPASSES, OptShredIntPasses);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_SHREDINTUSEGUTMANN, OptShredIntGutmann);
  WriteBool(SECTION_SECURETRAYUTIL, KEY_SHREDINTSHOWPROGRESS, OptShredIntShowProgress);

  WriteInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFREESPCFILESZ, OptShredIntFreeSpcFileSize);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFREESPCFILECREATIONBLKSZ, OptShredIntFreeSpcFileCreationBlkSize);
  WriteInteger(SECTION_SECURETRAYUTIL, KEY_SHREDINTFILEBUFSZ, OptShredIntFileBufferSize);

  WriteString(SECTION_SCRAMDISK, KEY_SCRAMDISKEXE, OptOTFScramDiskExe);
  WriteBool(SECTION_SCRAMDISK, KEY_USEREDSCREEN, OptOTFScramDiskUseRedScreen);
  WriteBool(SECTION_SCRAMDISK, KEY_DISABLESVLPROTECTION, OptOTFScramDiskDisableSVLProtection);
  WriteString(SECTION_SCRAMDISK, KEY_SCRAMDISKPREFDRIVE, OptOTFScramDiskPrefDrive);

  WriteString(SECTION_OTFCROSSCRYPT, KEY_CROSSCRYPTEXE, OptOTFCrossCryptExe);

  WriteString(SECTION_OTFE4M, KEY_E4MEXE, OptOTFE4MExe);
  WriteBool(SECTION_OTFE4M, KEY_E4MMOUNTDEVICEDLG9XSTYLE, OptOTFE4MMountDeviceDlg9xStyle);
  WriteBool(SECTION_OTFE4M, KEY_SUPPORTSCRAMDISK, OptOTFE4MSupportScramDisk);

  WriteString(SECTION_OTFFREEOTFE, KEY_FREEOTFEEXE, OptOTFFreeOTFEExe);

  // BEFORE CHANGING THIS!!!
  // These numbers need to be kept the same - otherwise the user's options will
  // mysteriously change
  case OptOTFTrueCryptVersionHint of
    tcvAuto: tempInt := 1;
    tcv21  : tempInt := 2;
    tcv21a : tempInt := 3;
  else
    tempInt := 0; // DblClkSlotInfo
  end;
  WriteInteger(SECTION_OTFTRUECRYPT, KEY_VERSIONHINT, tempInt);
  WriteBool(SECTION_OTFTRUECRYPT, KEY_TRUECRYPTMOUNTDEVICEDLG9XSTYLE, OptOTFTrueCryptMountDeviceDlg9xStyle);


  Result := TRUE;
end;

function TSTUIniFile.WriteIniFileSelfD() : boolean;
var
  tempInt : integer;
  i : integer;
  keyFilename : string;
begin
  WriteBool(SECTION_SELFD, KEY_FACILITIESENABLE1, OptSelfDFacilitiesEnable1);
  WriteBool(SECTION_SELFD, KEY_FACILITIESENABLE2, OptSelfDFacilitiesEnable2);

  // BEFORE CHANGING THIS!!!
  // These numbers need to be kept the same - otherwise the user's options will
  // mysteriously change
  case OptSelfDMenuitemLocation of
    sdmiTop   : tempInt := 1;
    sdmiBottom: tempInt := 2;
  else
    tempInt := 0; // Default and in case of error
  end;
  WriteInteger(SECTION_SELFD, KEY_MENUTRIGGERLOCATION, tempInt);
  WriteBool(SECTION_SELFD, KEY_ENABLEMENUTRIGGER, OptSelfDEnableMenuitem);

  WriteBool(SECTION_SELFD, KEY_ENABLESELFDHOTKEY, OptSelfDEnableHotkey);
  WriteString(SECTION_SELFD, KEY_SELFDHOTKEY, OptSelfDHotkey);
  WriteBool(SECTION_SELFD, KEY_ENABLESERIALTRIGGER, OptSelfDTriggerPortEnable);
  WriteString(SECTION_SELFD, KEY_SERIALTRIGGERPORT, OptSelfDTriggerPort);
  WriteInteger(SECTION_SELFD, KEY_SERIALTRIGGERINTERVAL, OptSelfDTriggerPortInterval);
  WriteBool(SECTION_SELFD, KEY_ENABLEINTERNETTRIGGER, OptSelfDEnableInternetTrigger);
  WriteInteger(SECTION_SELFD, KEY_INTERNETTRIGGERPORT, OptSelfDInternetPort);
  WriteInteger(SECTION_SELFD, KEY_INTERNETTRIGGERCRPAIRS, OptSelfDInternetCRPairs);

  for i:=1 to OptSelfDInternetCRPairs do
    begin
    WriteString(SECTION_SELFD, KEY_INTERNETTRIGGERCHAL+inttostr(i), OptSelfDInternetChal[i-1]);
    WriteString(SECTION_SELFD, KEY_INTERNETTRIGGERRESP+inttostr(i), OptSelfDInternetResp[i-1]);
    end;

  WriteBool(SECTION_SELFD, KEY_TRIGGERTESTMODE, OptSelfDTriggerTestMode);

  WriteBool(SECTION_SELFD, KEY_DESTROYMOUNTEDENABLE, OptSelfDDestroyMountedEnable);
  WriteBool(SECTION_SELFD, KEY_DESTROYMOUNTEDQUICK, OptSelfDDestroyMountedQuickShred);
  WriteBool(SECTION_SELFD, KEY_RUNADDITIONALENABLE, OptSelfDRunAdditionalEnable);
  WriteString(SECTION_SELFD, KEY_RUNADDITIONALFILENAME, OptSelfDRunAdditionalFilename);
  WriteBool(SECTION_SELFD, KEY_STEALTHSELFD, OptSelfDStealth);

  WriteBool(SECTION_SELFD, KEY_RANDOMIZECLOCK, OptSelfDRandomizeDT);
  WriteString(SECTION_SELFD, KEY_RANDOMIZECLOCKSTARTDATETIME, DateTimeToStr(OptSelfDRandomizeDTStart));
  WriteString(SECTION_SELFD, KEY_RANDOMIZECLOCKENDDATETIME, DateTimeToStr(OptSelfDRandomizeDTEnd));
  WriteBool(SECTION_SELFD, KEY_RANDOMIZECLOCKSTARTCURRENT, OptSelfDRandomizeDTStartCurrent);
  WriteBool(SECTION_SELFD, KEY_RANDOMIZECLOCKENDCURRENT, OptSelfDRandomizeDTEndCurrent);

  // Last, write out the target files
  WriteInteger(SECTION_SELFDTARGETS, KEY_TARGETSCOUNT, OptSelfDTargetFiles.count);
  for i:=1 to OptSelfDTargetFiles.count do
    begin
    keyFilename := KEY_TARGET + inttostr(i-1);
    WriteString(SECTION_SELFDTARGETS, keyFilename, OptSelfDTargetFiles[i-1]);
    end;

  Result := TRUE;
end;

function TSTUIniFile.WriteIniFileShellExt() : boolean;
begin
  WriteBool(SECTION_SHELLEXT, KEY_SUBMENU, OptShellExtSubMenu);
  WriteString(SECTION_SHELLEXT, KEY_SUBMENUTITLE, OptShellExtSubMenuTitle);
  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_SEPARATORS, OptShellExtMenuSeparators);

  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_MOUNTVOLUME, OptShellExtMenuMountVolume);
  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_SHREDFILE, OptShellExtMenuShredFile);
  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_SHREDDIR, OptShellExtMenuShredDir);
  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_SHREDFREESPACE, OptShellExtMenuShredFreeSpace);
  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_TOUCH, OptShellExtMenuTouch);
  WriteBool(SECTION_SHELLEXT, KEY_MENUITEM_HASH, OptShellExtMenuHash);
  WriteBool(SECTION_SHELLEXT, KEY_APPPATH, OptShellExtAppPath);
  Result := TRUE;
end;

function TSTUIniFile.WriteIniFileHash() : boolean;
var
  hashLoop: fhHashType;
  currKey: string;
  hashUnified: THashAlgUnified;
begin
  hashUnified:= THashAlgUnified.Create(nil);
  try
    for hashLoop:=low(hashLoop) to high(hashLoop) do
      begin
      hashUnified.ActiveHash := hashLoop;
      currKey := KEY_HASH_ENABLEHASH + hashUnified.Title;
      WriteBool(SECTION_HASH, currKey, OptHashEnabled[hashLoop]);
      end;
  finally
    hashUnified.Free();
  end;

  WriteBool(SECTION_HASH, KEY_HASH_UPPERCASE, OptHashInUppercase);

  Result := TRUE;
end;

function TSTUIniFile.WriteIniFileOTFCryptoSupport() : boolean;
var
  OTFSystemLoop: TOTFESystem;
  currKey: string;
  tempInt: integer;
begin
  for OTFSystemLoop:=low(OptOTFCryptoSupport) to high(OptOTFCryptoSupport) do
    begin
    currKey := KEY_OTFCS_SUPPORT + OTFESInternalNames[OTFSystemLoop];
    WriteBool(SECTION_OTFCRYPTOSYSTEMS, currKey, OptOTFCryptoSupport[OTFSystemLoop]);
    end;

  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DEFAULTSYSTEM, OTFESInternalNames[OptOTFCryptoDefault]);
  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DEFAULT_PROMPT_DIR, OptOTFDefaultBrowseDir);
  WriteInteger(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DEFAULT_PROMPT_FILTER, OptOTFDefaultBrowseFilter);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_EXPLOREAFTERMOUNT, OptOTFExploreAfterMount);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_MOUNTVOLSTART, OptOTFMountVolStart);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_MOUNTPARTSTART, OptOTFMountPartStart);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_ALWAYSUNMOUNTEDICON, OptOTFAlwaysUnmountedIcon);

  case OptOTFOnExit of
    ExitDismount       : tempInt := 1;
    ExitDismountBrutal : tempInt := 2;
  else
    tempInt := 0 // ExitNone
  end;
  WriteInteger(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_ONEXIT, tempInt);

  // BEFORE CHANGING THIS!!!
  // These numbers need to be kept the same - otherwise the user's options will
  // mysteriously change
  case OptOTFOnDblClk of
    DblClkDismount       : tempInt := 1;
    DblClkDismountBrutal : tempInt := 2;
    DblClkOpenPromptDismount       : tempInt := 3;
    DblClkOpenPromptDismountBrutal : tempInt := 4;
    DblClkOpenPromptDismountPrompt : tempInt := 5;
    DblClkOpenPrompt               : tempInt := 6;
    DblClkDismountPrompt           : tempInt := 7;
  else
    tempInt := 0; // DblClkSlotInfo
  end;
  WriteInteger(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_ONDBLCLK, tempInt);

  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTENABLE, OptOTFHKeyDismountEnable);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTBRUTALENABLE, OptOTFHKeyDismountBrutalEnable);
  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTHOTKEY, OptOTFHKeyDismountKey);
  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTBRUTALHOTKEY, OptOTFHKeyDismountBrutalKey);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_HIDEPASSWORDSENTEREDCK, OptOTFHidePasswordsEnteredCk);

  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_POSTMOUNT, OptOTFPostMountExe);
  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_PREDISMOUNT, OptOTFPreDismountExe);
  WriteString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_POSTDISMOUNT, OptOTFPostDismountExe);
  WriteBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_WARNNOTFOUND, OptOTFPrePostWarnNotFound);


  Result := TRUE;
end;

function TSTUIniFile.ReadIniFileOTFCryptoSupport() : boolean;
var
  OTFSystemLoop: TOTFESystem;
  currKey: string;
  intrnlNameDeflt: string;
  tempInt: integer;
begin
  for OTFSystemLoop:=low(OptOTFCryptoSupport) to high(OptOTFCryptoSupport) do
    begin
    currKey := KEY_OTFCS_SUPPORT + OTFESInternalNames[OTFSystemLoop];
    OptOTFCryptoSupport[OTFSystemLoop] := ReadBool(SECTION_OTFCRYPTOSYSTEMS, currKey, DFLT_KEY_OTFCS_SUPPORT);
    end;

  intrnlNameDeflt := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DEFAULTSYSTEM, DFLT_KEY_OTFCS_DEFAULTSYSTEM);
  for OTFSystemLoop:=low(OTFESInternalNames) to high(OTFESInternalNames) do
    begin
    if intrnlNameDeflt=OTFESInternalNames[OTFSystemLoop] then
      begin
      OptOTFCryptoDefault := OTFSystemLoop;
      end;
    end;

  OptOTFDefaultBrowseDir := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DEFAULT_PROMPT_DIR, DFLT_KEY_OTFCS_DEFAULT_PROMPT_DIR);
  OptOTFDefaultBrowseFilter := ReadInteger(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DEFAULT_PROMPT_FILTER, DFLT_KEY_OTFCS_DEFAULT_PROMPT_FILTER);
  OptOTFExploreAfterMount := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_EXPLOREAFTERMOUNT, DFLT_KEY_OTFCS_EXPLOREAFTERMOUNT);
  OptOTFMountVolStart := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_MOUNTVOLSTART, DFLT_KEY_OTFCS_MOUNTVOLSTART);
  OptOTFMountPartStart := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_MOUNTPARTSTART, DFLT_KEY_OTFCS_MOUNTPARTSTART);
  OptOTFAlwaysUnmountedIcon := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_ALWAYSUNMOUNTEDICON, DFLT_KEY_OTFCS_ALWAYSUNMOUNTEDICON);

  tempInt := ReadInteger(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_ONEXIT, DFLT_KEY_OTFCS_ONEXIT);
  case tempInt of
    1 : OptOTFOnExit := ExitDismount;
    2 : OptOTFOnExit := ExitDismountBrutal;
  else
    OptOTFOnExit := ExitNone; // Default and in case of error
  end;

  // BEFORE CHANGING THIS!!!
  // These numbers need to be kept the same - otherwise the user's options will
  // mysteriously change
  tempInt := ReadInteger(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_ONDBLCLK, DFLT_KEY_OTFCS_ONDBLCLK);
  case tempInt of
    1 : OptOTFOnDblClk := DblClkDismount;
    2 : OptOTFOnDblClk := DblClkDismountBrutal;
    3 : OptOTFOnDblClk := DblClkOpenPromptDismount;
    4 : OptOTFOnDblClk := DblClkOpenPromptDismountBrutal;
    5 : OptOTFOnDblClk := DblClkOpenPromptDismountPrompt;
    6 : OptOTFOnDblClk := DblClkOpenPrompt;
    7 : OptOTFOnDblClk := DblClkDismountPrompt;
  else
    OptOTFOnDblClk := DblClkSlotInfo; // Default and in case of error
  end;

  OptOTFHKeyDismountEnable := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTENABLE, DFLT_KEY_OTFCS_DISMOUNTENABLE);
  OptOTFHKeyDismountBrutalEnable := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTBRUTALENABLE, DFLT_KEY_OTFCS_DISMOUNTBRUTALENABLE);
  OptOTFHKeyDismountKey := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTHOTKEY, DFLT_KEY_OTFCS_DISMOUNTHOTKEY);
  OptOTFHKeyDismountBrutalKey := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_DISMOUNTBRUTALHOTKEY, DFLT_KEY_OTFCS_DISMOUNTBRUTALHOTKEY);
  OptOTFHidePasswordsEnteredCk := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_HIDEPASSWORDSENTEREDCK, DFLT_KEY_OTFCS_HIDEPASSWORDSENTEREDCK);

  OptOTFPostMountExe    := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_POSTMOUNT, DFLT_KEY_OTFCS_PREPOSTEXE_POSTMOUNT);
  OptOTFPreDismountExe  := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_PREDISMOUNT, DFLT_KEY_OTFCS_PREPOSTEXE_PREDISMOUNT);
  OptOTFPostDismountExe := ReadString(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_POSTDISMOUNT, DFLT_KEY_OTFCS_PREPOSTEXE_POSTDISMOUNT);
  OptOTFPrePostWarnNotFound := ReadBool(SECTION_OTFCRYPTOSYSTEMS, KEY_OTFCS_PREPOSTEXE_WARNNOTFOUND, DFLT_KEY_OTFCS_PREPOSTEXE_WARNNOTFOUND);


  Result := TRUE;
end;

function TSTUIniFile.ReadTouchFiles(): boolean;
var
  count: integer;
  i: integer;
  currentTarget: TTouchFileTarget;
  keyTgtFilename: string;
  keyTgtDateTime: string;
  keyTgtUseCurrent: string;
begin
  Result := TRUE;

  count:= ReadInteger(SECTION_TOUCHFILES, KEY_TOUCHFILESCOUNT, DFLT_KEY_TOUCHFILESCOUNT);
  ClearTouchTargetFilesList();
  for i:=1 to count do
    begin
    currentTarget:= TTouchFileTarget.create();
    keyTgtFilename   := KEY_TOUCHFILESTGT + inttostr(i-1);
    keyTgtDateTime   := KEY_TOUCHFILESTGTDATETIME + inttostr(i-1);
    keyTgtUseCurrent := KEY_TOUCHFILESTGTUSECURRENTDATEDATETIME + inttostr(i-1);

    currentTarget.filename := ReadString(SECTION_TOUCHFILES, keyTgtFilename, DFLT_KEY_TOUCHFILESTGT);
    currentTarget.dateTime := strtodatetime(ReadString(SECTION_TOUCHFILES, keyTgtDateTime, DFLT_KEY_TOUCHFILESTGTDATETIME));
    currentTarget.useCurrentDateTime := ReadBool(SECTION_TOUCHFILES, keyTgtUseCurrent, DFLT_KEY_TOUCHFILESTGTCURRENTDATEDATETIME);

    OptTouchTargetFiles.addobject(currentTarget.filename, currentTarget);
    end;

    OptTouchFrequency := ReadInteger(SECTION_TOUCHFILES, KEY_TOUCHFILESFREQUENCY, DFLT_KEY_TOUCHFILESFREQUENCY);
    OptTouchRegularly := ReadBool(SECTION_TOUCHFILES, KEY_TOUCHFILESREGULARLY, DFLT_KEY_TOUCHFILESREGULARLY);
    OptTouchOnStart   := ReadBool(SECTION_TOUCHFILES, KEY_TOUCHFILESONSTART, DFLT_KEY_TOUCHFILESONSTART);
    OptTouchOnExit    := ReadBool(SECTION_TOUCHFILES, KEY_TOUCHFILESONEXIT, DFLT_KEY_TOUCHFILESONEXIT);

end;

function TSTUIniFile.WriteTouchFiles(): boolean;
var
  i: integer;
  currentTarget: TTouchFileTarget;
  keyTgtFilename: string;
  keyTgtDateTime: string;
  keyTgtUseCurrent: string;
begin
  Result := TRUE;

  WriteInteger(SECTION_TOUCHFILES, KEY_TOUCHFILESCOUNT, OptTouchTargetFiles.count);

  for i:=1 to OptTouchTargetFiles.count do
    begin
    keyTgtFilename   := KEY_TOUCHFILESTGT + inttostr(i-1);
    keyTgtDateTime   := KEY_TOUCHFILESTGTDATETIME + inttostr(i-1);
    keyTgtUseCurrent := KEY_TOUCHFILESTGTUSECURRENTDATEDATETIME + inttostr(i-1);

    currentTarget := TTouchFileTarget(OptTouchTargetFiles.objects[i-1]);

    WriteString(SECTION_TOUCHFILES, keyTgtFilename, currentTarget.filename);
    WriteString(SECTION_TOUCHFILES, keyTgtDateTime, datetimetostr(currentTarget.dateTime));
    WriteBool(SECTION_TOUCHFILES, keyTgtUseCurrent, currentTarget.useCurrentDateTime);
    end;

  WriteInteger(SECTION_TOUCHFILES, KEY_TOUCHFILESFREQUENCY, OptTouchFrequency);
  WriteBool(SECTION_TOUCHFILES, KEY_TOUCHFILESREGULARLY, OptTouchRegularly);
  WriteBool(SECTION_TOUCHFILES, KEY_TOUCHFILESONSTART, OptTouchOnStart);
  WriteBool(SECTION_TOUCHFILES, KEY_TOUCHFILESONEXIT, OptTouchOnExit);

end;


procedure TSTUIniFile.ClearTouchTargetFilesList();
var
  i: integer;
begin
  for i:=0 to (OptTouchTargetFiles.count-1) do
    begin
    TTouchFileTarget(OptTouchTargetFiles.objects[i]).Free();
    end;

  OptTouchTargetFiles.Clear();
end;


procedure TSTUIniFile.CheckVersionID();
var
  oldVersionStr: string;
  upgradeMessage: string;

  majorVersion   : integer; // Only used to build up oldVersionStr
  minorVersion   : integer; // Only used to build up oldVersionStr
  revisionVersion: integer; // Only used to build up oldVersionStr
  buildVersion   : integer; // Only used to build up oldVersionStr

  thisMajorVersion   : integer;
  thisMinorVersion   : integer;
  thisRevisionVersion: integer;
  thisBuildVersion   : integer;
  strThisVersion     : string;

  betaVersion: integer;
begin
  oldVersionStr    := ReadString(SECTION_SECURETRAYUTIL, KEY_VERSION, '-1.0');
  majorVersion     := ReadInteger(SECTION_SECURETRAYUTIL, KEY_MAJOR_VERSION, -1);
  minorVersion     := ReadInteger(SECTION_SECURETRAYUTIL, KEY_MINOR_VERSION, -1);
  revisionVersion  := ReadInteger(SECTION_SECURETRAYUTIL, KEY_RELEASE_VERSION, -1);
  buildVersion     := ReadInteger(SECTION_SECURETRAYUTIL, KEY_BUILD_VERSION, -1);
  betaVersion      := ReadInteger(SECTION_SECURETRAYUTIL, KEY_BETA_VERSION, -1);

  SDUGetVersionInfo('', thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion);
  strThisVersion := Format('%d.%.2d.%.2d.%.4d', [thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion]);
  if (APP_BETA_BUILD>-1) then
    begin
    strThisVersion := strThisVersion + ' BETA ' + inttostr(APP_BETA_BUILD);
    end;

  NewVersion := FALSE;

  if oldVersionStr='-1.0' then
    begin
    // Any message to be displayed the 1st time the user runs the application
    // to be placed here
    NewVersion := TRUE;
    end
  else if (oldVersionStr='1.00') OR
          (oldVersionStr='1.10') OR
          (majorVersion<thisMajorVersion) OR
          ((majorVersion=thisMajorVersion) AND (minorVersion<thisMinorVersion)) OR
          ((betaVersion<>-1) AND (not(APP_BETA_BUILD>-1))) OR
          ((betaVersion<>-1) AND (betaVersion<APP_BETA_BUILD)) then
    begin
    // The INI file dates back from an older version, so warn the user
    
    if not((oldVersionStr='1.00') or (oldVersionStr='1.10')) then
      begin
      oldVersionStr := Format('%d.%.2d.%.2d.%.4d', [majorVersion, minorVersion, revisionVersion, buildVersion]);
      end;

    if betaVersion>-1 then
      begin
      oldVersionStr := oldVersionStr + ' BETA ' + inttostr(betaVersion);
      end;

    upgradeMessage := 'It appears as though your "SecureTrayUtil.ini" file dates back to v'+oldVersionStr;
    upgradeMessage := upgradeMessage+CRLF+CRLF+'You may find that some of your options mysteriously revert back to their defaults when running this version (v'+strThisVersion+').';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'This is "normal", and due to two things:';
    upgradeMessage := upgradeMessage+CRLF+'1) the format of the ini file has changed to make it more logical and to take into account improvements in the software';
    upgradeMessage := upgradeMessage+CRLF+'2) lazyness on the part of the author (it wouldn''t be difficult to write a converter, but it''s just too much hassle)';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'To prevent this dialog from reappearing, either delete your existing SecureTrayUtil.ini file (located in your Windows directory) or goto any of the options screens, change an option, change it back, and click "OK" or "Apply"';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'It is recommended that you check your option settings to ensure correct operation.';

    messagedlg(upgradeMessage, mtWarning, [mbOK], 0);

    NewVersion := TRUE;
    end;

end;


END.

