unit CalculateFileHash_U;
// Description: File/Test Vector Hash Dialog
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  HashAlg_U,
  HashAlgUnified_U,
  AppGlobalConsts_U;

type
  TCalculateFileHash_F = class(TForm)
    lblMD5: TLabel;
    edMD5: TEdit;
    edSHA1: TEdit;
    lblSHA1: TLabel;
    pbClose: TButton;
    Label3: TLabel;
    pbBrowse: TButton;
    OpenDialog: TOpenDialog;
    pbTest: TButton;
    edFilename: TEdit;
    pbHashFile: TButton;
    edSHA: TEdit;
    lblSHA: TLabel;
    pbCancel: TButton;
    edMD4: TEdit;
    lblMD4: TLabel;
    edRIPEMD128: TEdit;
    lblRIPEMD128: TLabel;
    edMD2: TEdit;
    lblMD2: TLabel;
    lblGOST: TLabel;
    lblRIPEMD320: TLabel;
    edRIPEMD160: TEdit;
    lblRIPEMD160: TLabel;
    lblRIPEMD256: TLabel;
    meRIPEMD256: TMemo;
    meRIPEMD320: TMemo;
    meGOST: TMemo;
    lblSHA256: TLabel;
    lblSHA384: TLabel;
    lblSHA512: TLabel;
    meSHA512: TMemo;
    meSHA384: TMemo;
    meSHA256: TMemo;
    meTiger: TMemo;
    lblTiger: TLabel;
    procedure pbCloseClick(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure pbTestClick(Sender: TObject);
    procedure pbHashFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
  private
    hashObj: THashAlgUnified;

    procedure CalculateAll(asTestString: boolean);
    function  GetHashDisplay(hashType: fhHashType): TControl;
    function  GetHashLabel(hashType: fhHashType): TControl;
    procedure EnableDisableDisplays();
    procedure ClearHashDisplays();
    procedure EnableDisableControls(enable: boolean);
  public
    HashesEnabled: array [fhHashType] of boolean;
    procedure HashFile(filename: string);
  end;


implementation

{$R *.DFM}

uses SDUGeneral,
     AppGlobal_U;

procedure TCalculateFileHash_F.pbCloseClick(Sender: TObject);
begin
  Close();

end;


procedure TCalculateFileHash_F.pbBrowseClick(Sender: TObject);
begin
  OpenDialog.title := 'Hash File';
  SDUOpenSaveDialogSetup(OpenDialog, edFilename.text);
  if OpenDialog.Execute then
    begin
    edFilename.text := OpenDialog.filename;
    end;

end;


procedure TCalculateFileHash_F.pbTestClick(Sender: TObject);
begin
  CalculateAll(TRUE);

end;

procedure TCalculateFileHash_F.CalculateAll(asTestString: boolean);
var
  i: fhHashType;
  hashDisp: TControl;
  strHash: string;
  hash: THashArray;
begin
  Application.ProcessMessages();
  if self.visible then
    begin
    Screen.Cursor := crHourGlass;
    end;

  EnableDisableControls(FALSE);

  ClearHashDisplays();

  try
    pbCancel.tag := 0;
    pbClose.Cancel  := FALSE;
    pbCancel.Cancel := TRUE;

    for i:=low(fhHashType) to high(fhHashType) do
      begin
      if pbCancel.tag=0 then
        begin
        if HashesEnabled[i] then
          begin
          hashDisp := GetHashDisplay(i);
          TEdit(hashDisp).text := 'Calculating...';
          Application.ProcessMessages();
          hashObj.ActiveHash := i;
          if asTestString then
            begin
            hash := hashObj.HashString(edFilename.text);
            end
          else
            begin
            hashObj.HashFile(edFilename.text, hash)
            end;

          strHash := hashObj.HashToDisplay(hash);
          if STU_INI_FILE.OptHashInUppercase then
            begin
            strHash := uppercase(strHash);
            end
          else
            begin
            strHash := lowercase(strHash);
            end;

          TEdit(hashDisp).text := strHash;

          Application.ProcessMessages();

          end; // if HashesEnabled[i] then
        end; // if pbCancel.tag=0 then
      end; // for i:=low(hashObjects) to high(hashObjects) do

  finally
    if self.visible then
      begin
      Screen.Cursor := crDefault;
      end;

    pbClose.Cancel  := TRUE;
    pbCancel.Cancel := FALSE;

    EnableDisableControls(TRUE);
    
  end;

end;

procedure TCalculateFileHash_F.pbHashFileClick(Sender: TObject);
begin
  HashFile(edFilename.text);

end;

procedure TCalculateFileHash_F.FormCreate(Sender: TObject);
var
  i: fhHashType;
begin
  for i:=low(fhHashType) to high(fhHashType) do
    begin
    HashesEnabled[i] := TRUE;
    end;

  hashObj:= THashAlgUnified.Create(nil);

end;

procedure TCalculateFileHash_F.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  hashObj.Free();

end;

function TCalculateFileHash_F.GetHashDisplay(hashType: fhHashType): TControl;
begin
  Result := nil;

  case hashType of
    hashMD2 : Result := edMD2;
    hashMD4 : Result := edMD4;
    hashMD5 : Result := edMD5;
    hashSHA : Result := edSHA;
    hashSHA1: Result := edSHA1;
    hashSHA256: Result := meSHA256;
    hashSHA384: Result := meSHA384;
    hashSHA512: Result := meSHA512;
    hashRIPEMD128 : Result := edRIPEMD128;
    hashRIPEMD160 : Result := edRIPEMD160;
    hashRIPEMD256 : Result := meRIPEMD256;
    hashRIPEMD320 : Result := meRIPEMD320;
    hashGOST : Result := meGOST;
    hashTiger : Result := meTiger;
  end;

end;

function TCalculateFileHash_F.GetHashLabel(hashType: fhHashType): TControl;
begin
  Result := nil;

  case hashType of
    hashMD2 : Result := lblMD2;
    hashMD4 : Result := lblMD4;
    hashMD5 : Result := lblMD5;
    hashSHA : Result := lblSHA;
    hashSHA1: Result := lblSHA1;
    hashSHA256: Result := lblSHA256;
    hashSHA384: Result := lblSHA384;
    hashSHA512: Result := lblSHA512;
    hashRIPEMD128: Result := lblRIPEMD128;
    hashRIPEMD160: Result := lblRIPEMD160;
    hashRIPEMD256: Result := lblRIPEMD256;
    hashRIPEMD320: Result := lblRIPEMD320;
    hashGOST: Result := lblGOST;
    hashTiger: Result := lblTiger;
  end;

end;

procedure TCalculateFileHash_F.FormShow(Sender: TObject);
begin
  EnableDisableDisplays();

end;

procedure TCalculateFileHash_F.EnableDisableDisplays();
var
  i: fhHashType;
begin
  for i:=low(fhHashType) to high(fhHashType) do
    begin
    if not(HashesEnabled[i]) then
      begin
      TEdit(GetHashDisplay(i)).Text := 'Disabled by user.';
      SDUEnableControl(GetHashDisplay(i), HashesEnabled[i]);
      SDUEnableControl(GetHashLabel(i), HashesEnabled[i]);
      end;

    end;

end;

procedure TCalculateFileHash_F.ClearHashDisplays();
var
  i: fhHashType;
begin
  for i:=low(fhHashType) to high(fhHashType) do
    begin
    if HashesEnabled[i] then
      begin
      TEdit(GetHashDisplay(i)).Text := '';
      end;

    end;

end;


procedure TCalculateFileHash_F.pbCancelClick(Sender: TObject);
begin
  showmessage('Hashing cancelled.');
  pbCancel.tag := 1;

end;

procedure TCalculateFileHash_F.HashFile(filename: string);
begin
  edFilename.text := filename;

  if FileExists(filename) then
    begin
    CalculateAll(FALSE);
    end
  else
    begin
    if filename='' then
      begin
      showmessage('Please enter the filename of the file to be hashed.');
      end
    else
      begin
      showmessage('File "'+filename+'" not found.');
      end;

    ClearHashDisplays();
    FocusControl(edFilename);
    end;

end;

procedure TCalculateFileHash_F.EnableDisableControls(enable: boolean);
begin
  pbCancel.enabled   := not(enable);
  pbClose.enabled    := enable;
  pbHashFile.enabled := enable;
  pbTest.enabled     := enable;
  pbBrowse.enabled   := enable;
  SDUEnableControl(edFilename, enable);
end;

END.


