unit ConfigShredderExt_U;
// Description: External (3rd party) shredder config dialog
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TConfigShredderExt_F = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    pbApply: TButton;
    pcOptions: TPageControl;
    tsFilesDirs: TTabSheet;
    GroupBox1: TGroupBox;
    Label2: TLabel;
    Label7: TLabel;
    edExtShredFile: TEdit;
    pbBrExtFileShred: TButton;
    OpenDialog: TOpenDialog;
    Label4: TLabel;
    Label5: TLabel;
    edExtShredDir: TEdit;
    pbBrExtDirShred: TButton;
    ckDirsDifferentExt: TCheckBox;
    ckFilesBeforeDir: TCheckBox;
    Label1: TLabel;
    Label3: TLabel;
    tsFreeSpace: TTabSheet;
    GroupBox2: TGroupBox;
    Label6: TLabel;
    Label8: TLabel;
    edExtShredFreeSpace: TEdit;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure pbBrExtFileShredClick(Sender: TObject);
    procedure pbBrExtDirShredClick(Sender: TObject);
    procedure pbApplyClick(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure SetApplyButton(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    procedure LoadINIValues();
    procedure EnableDisableAllControls();
  public
    { Public declarations }
  end;

var
  ConfigShredderExt_F: TConfigShredderExt_F;

implementation

{$R *.DFM}

uses AppGlobal_U, AppGlobalConsts_U,
     STUIniFile_U, SDUGeneral;

procedure TConfigShredderExt_F.FormCreate(Sender: TObject);
begin
  LoadINIValues();

  pcOptions.activepage := tsFilesDirs;

end;

procedure TConfigShredderExt_F.pbBrExtFileShredClick(Sender: TObject);
begin
  OpenDialog.title := 'Locate Shredder Executable';
  OpenDialog.filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edExtShredFile.text);
  if OpenDialog.execute then
    begin
    edExtShredFile.text := OpenDialog.filename + ' %s';
    end;

end;

procedure TConfigShredderExt_F.pbBrExtDirShredClick(Sender: TObject);
begin
  OpenDialog.title := 'Locate Shredder Executable';
  OpenDialog.filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edExtShredDir.text);
  if OpenDialog.execute then
    begin
    edExtShredDir.text := OpenDialog.filename + ' %s';
    end;

end;

procedure TConfigShredderExt_F.pbApplyClick(Sender: TObject);
begin
  // General Page...
  STU_INI_FILE.OptShredExtFileExe := edExtShredFile.text;
  STU_INI_FILE.OptShredExtDirExe := edExtShredDir.text;
  STU_INI_FILE.OptShredExtFreeSpaceExe := edExtShredFreeSpace.text;

  STU_INI_FILE.OptShredExtDirDiff := ckDirsDifferentExt.checked;
  STU_INI_FILE.OptShredExtShredFilesThenDir := ckFilesBeforeDir.checked;

  // Finish off...
  STU_INI_FILE.WriteIniFile;
  pbApply.enabled := FALSE;

  LoadINIValues();

end;

procedure TConfigShredderExt_F.LoadINIValues();
begin
  STU_INI_FILE.ReadIniFile;

  // General page...
  edExtShredFile.text := STU_INI_FILE.OptShredExtFileExe;
  edExtShredDir.text := STU_INI_FILE.OptShredExtDirExe;
  edExtShredFreeSpace.text := STU_INI_FILE.OptShredExtFreeSpaceExe;

  ckDirsDifferentExt.checked := STU_INI_FILE.OptShredExtDirDiff;
  ckFilesBeforeDir.checked := STU_INI_FILE.OptShredExtShredFilesThenDir;

  // Finish off...
  pbApply.enabled := FALSE;
end;

procedure TConfigShredderExt_F.pbOKClick(Sender: TObject);
begin
  if pbApply.enabled then
    begin
    pbApplyClick(sender);
    end;

  modalresult := mrOK;

end;

procedure TConfigShredderExt_F.EnableDisableAllControls();
var
  filesBeforeDir: boolean;
begin
  SDUEnableControl(edExtShredDir, ckDirsDifferentExt.checked);
  SDUEnableControl(pbBrExtDirShred, ckDirsDifferentExt.checked);

  filesBeforeDir := (ckDirsDifferentExt.checked) AND (edExtShredDir.text<>'');
  SDUEnableControl(ckFilesBeforeDir, filesBeforeDir);

end;


procedure TConfigShredderExt_F.SetApplyButton(Sender: TObject);
begin
  EnableDisableAllControls();
  pbApply.enabled := TRUE;

end;

procedure TConfigShredderExt_F.Button1Click(Sender: TObject);
begin
  OpenDialog.title := 'Locate Free Space Shredder Executable';
  OpenDialog.filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edExtShredFreeSpace.text);
  if OpenDialog.execute then
    begin
    edExtShredFreeSpace.text := OpenDialog.filename + ' %c';
    end;

end;

END.

