unit ConfigShredderInt_U;
// Description: Internal shredder config dialog
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ComCtrls, ExtCtrls;

type
  TConfigShredderInt_F = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    pbApply: TButton;
    pcOptions: TPageControl;
    tsGeneral: TTabSheet;
    GroupBox3: TGroupBox;
    ckShredSlack: TCheckBox;
    tsOverwriteAlgorithm: TTabSheet;
    GroupBox1: TGroupBox;
    rbUseGutmann: TRadioButton;
    rbNotUseGutmann: TRadioButton;
    Label10: TLabel;
    sePasses: TSpinEdit;
    Label4: TLabel;
    GroupBox2: TGroupBox;
    seShredBytes: TSpinEdit;
    Label1: TLabel;
    Label9: TLabel;
    Label3: TLabel;
    ckShowProgress: TCheckBox;
    Label2: TLabel;
    tsBuffers: TTabSheet;
    GroupBox4: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    seFreeSpcFileSize: TSpinEdit;
    seFileBufSize: TSpinEdit;
    seFreeSpcFileCreateBlkSize: TSpinEdit;
    Label7: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel4: TBevel;
    Label15: TLabel;
    Label16: TLabel;
    Label8: TLabel;
    Label14: TLabel;
    Label17: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure pbApplyClick(Sender: TObject);
    procedure SetApplyButton(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
  private
    procedure LoadINIValues();
    procedure EnableDisableAllControls();
  public
    { Public declarations }
  end;

var
  ConfigShredderInt_F: TConfigShredderInt_F;

implementation

{$R *.DFM}

uses AppGlobal_U, STUIniFile_U, SDUGeneral;

procedure TConfigShredderInt_F.FormCreate(Sender: TObject);
begin
  LoadINIValues();
  pcOptions.activepage := tsGeneral;
  EnableDisableAllControls();

end;

procedure TConfigShredderInt_F.pbApplyClick(Sender: TObject);
begin
  // General page...
  STU_INI_FILE.OptShredSlack := ckShredSlack.checked;
  STU_INI_FILE.OptShredIntShowProgress := ckShowProgress.checked;
  STU_INI_FILE.OptShredIntFirstBytes := seShredBytes.value;

  // Overwrite Algorithm page...
  STU_INI_FILE.OptShredIntGutmann := rbUseGutmann.checked;
  STU_INI_FILE.OptShredIntPasses := sePasses.value;

  // Buffers page...
  STU_INI_FILE.OptShredIntFreeSpcFileSize  := seFreeSpcFileSize.value;
  STU_INI_FILE.OptShredIntFreeSpcFileCreationBlkSize:= seFreeSpcFileCreateBlkSize.value;
  STU_INI_FILE.OptShredIntFileBufferSize   := seFileBufSize.value;

  // Finish off...
  STU_INI_FILE.WriteIniFile;
  pbApply.enabled := FALSE;

  LoadINIValues();
end;

procedure TConfigShredderInt_F.LoadINIValues();
begin
  STU_INI_FILE.ReadIniFile;

  // General page...
  ckShredSlack.checked := STU_INI_FILE.OptShredSlack;
  ckShowProgress.checked := STU_INI_FILE.OptShredIntShowProgress;
  seShredBytes.value := STU_INI_FILE.OptShredIntFirstBytes;

  // Overwrite Algorithm page...
  rbUseGutmann.checked := STU_INI_FILE.OptShredIntGutmann;
  rbNotUseGutmann.checked := not(STU_INI_FILE.OptShredIntGutmann);
  sePasses.value := STU_INI_FILE.OptShredIntPasses;

  // Buffers page...
  seFreeSpcFileSize.value := STU_INI_FILE.OptShredIntFreeSpcFileSize;
  seFreeSpcFileCreateBlkSize.value := STU_INI_FILE.OptShredIntFreeSpcFileCreationBlkSize;
  seFileBufSize.value := STU_INI_FILE.OptShredIntFileBufferSize;

  // Finish off...
  pbApply.enabled := FALSE;

end;


procedure TConfigShredderInt_F.SetApplyButton(Sender: TObject);
begin
  EnableDisableAllControls();
  pbApply.enabled := TRUE;

end;

procedure TConfigShredderInt_F.pbOKClick(Sender: TObject);
begin
  if pbApply.enabled then
    begin
    pbApplyClick(sender);
    end;

  if not(pbApply.enabled) then
    begin
    modalresult := mrOK;
    end;

end;

procedure TConfigShredderInt_F.EnableDisableAllControls();
begin
  SDUEnableControl(sePasses, rbNotUseGutmann.checked);

end;

END.

