unit OTFVolumeDetailsBestCrypt_U;
// Description:
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  OTFVolumeDetails_U,
  OTFEBestCryptStructures_U;

type
  TOTFVolumeDetailsBestCrypt_F = class(TOTFVolumeDetails_F)
    Label1021: TLabel;
    gbVolInfo: TGroupBox;
    Label1015: TLabel;
    Label1001: TLabel;
    Label1016: TLabel;
    Label1018: TLabel;
    Label1019: TLabel;
    reDescription: TRichEdit;
    edReadOnly: TEdit;
    edAlgorithm: TEdit;
    edKeyGen: TEdit;
    edErrsInMounting: TEdit;
    edFilename: TEdit;
    Label2023: TLabel;
    edVolLabel: TEdit;
    Label9000: TLabel;
    edDriveMountedAs: TEdit;
    Label2022: TLabel;
  private
    procedure DisplayActualVolumeInfo(info: TBCDiskInfo);
  public
    constructor Create(AOwner: TComponent); override;
    function  DisplayInfoForDrive(driveLetter: char): boolean; override;
  end;

implementation

{$R *.DFM}

uses
  OTFEBestCrypt_U,
  OTFEUnified_U,
  SDUGeneral;

constructor TOTFVolumeDetailsBestCrypt_F.Create(AOwner: TComponent);
begin
  inherited;

  OTFSystem:= otfesBestCrypt;

end;

function TOTFVolumeDetailsBestCrypt_F.DisplayInfoForDrive(driveLetter: char): boolean;
var
  info: TBCDiskInfo;
begin
  info := TBCDiskInfo.Create();
  try
    TOTFEBestCrypt(OTFComponent).GetVolumeInfo(driveLetter, info);
    DisplayActualVolumeInfo(info);

  finally
    info.Free();
  end;

  Result := TRUE;

end;

procedure TOTFVolumeDetailsBestCrypt_F.DisplayActualVolumeInfo(info: TBCDiskInfo);
var
  algorithmProb: boolean;
begin
  edDriveMountedAs.text := '<not mounted>';
  edReadOnly.text := '<not mounted>';
  edFilename.text := '<not mounted>';

  if info.volumeFilename<>'' then
    begin
    edFilename.text := info.volumeFilename;
    end;

  if info.mountedAs<>#0 then
    begin
    edDriveMountedAs.text := info.mountedAs + ':';
    edVolLabel.text := SDUVolumeID(info.mountedAs);
    if info.readOnly then
      begin
      edReadOnly.text := 'Readonly';
      end
    else
      begin
      edReadOnly.text := 'Read/Write';
      end;

    end;

{
These next three are no use to the user, so this block is commented out
  edExtent.text := '0x'+inttohex(info.extent, 2);
  edVersion.text := '0x'+inttohex(info.version, 2);
  edFileSystemID.text := '0x'+inttohex(info.fileSystemId, 4);
}

  algorithmProb := FALSE;
  if info.algorithmName='' then
    begin
    algorithmProb := TRUE;
    edAlgorithm.text := '???';
    end
  else
    begin
    edAlgorithm.text := info.algorithmName;
    end;
  if info.algorithmKeyLen=-1 then
    begin
    algorithmProb := TRUE;
    edAlgorithm.text := edAlgorithm.text + ' (??? bit)';
    end
  else
    begin
    edAlgorithm.text := edAlgorithm.text+' ('+inttostr(info.algorithmKeyLen)+' bit)';
    end;
  if algorithmProb then
    begin
    edAlgorithm.text := edAlgorithm.text+' (ID=0x'+inttohex(info.algorithmId, 1)+')';
    end;
  if info.keyGenName='' then
    begin
    edKeyGen.text := '??? (0x'+inttostr(info.keyGenId)+')';
    end
  else
    begin
    edKeyGen.text := info.keyGenName;
    end;

  reDescription.text := info.description;
  edErrsInMounting.text := inttostr(info.errsInMounting);

end;

END.

