unit OTFVolumeDetailsFreeOTFE_U;
// Description:
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  OTFVolumeDetails_U;

type
  TOTFVolumeDetailsFreeOTFE_F = class(TOTFVolumeDetails_F)
    gbVolInfo: TGroupBox;
    Label2015: TLabel;
    Label2022: TLabel;
    Label2023: TLabel;
    edFilename: TEdit;
    edDriveMountedAs: TEdit;
    edReadOnly: TEdit;
    Label2020: TLabel;
    Label9000: TLabel;
    edVolLabel: TEdit;
    Label9105: TLabel;
    Label9106: TLabel;
    Label9107: TLabel;
    edDeviceName: TEdit;
    edIVCypher: TEdit;
    edIVHash: TEdit;
    edMainCypher: TEdit;
    Label9108: TLabel;
  private
    { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    function  DisplayInfoForDrive(driveLetter: char): boolean; override;
  end;

implementation

{$R *.DFM}

uses
  OTFEFreeOTFE_U,
  OTFEUnified_U,
  SDUGeneral,
  activex,
  comobj,
  OTFEFreeOTFE_DriverAPI;

constructor TOTFVolumeDetailsFreeOTFE_F.Create(AOwner: TComponent);
begin
  inherited;

  OTFSystem:= otfesFreeOTFE;

end;

function TOTFVolumeDetailsFreeOTFE_F.DisplayInfoForDrive(driveLetter: char): boolean;
var
  info: TOTFEFreeOTFEVolumeInfo;
  hashDetails: TFreeOTFEHash;
  cypherDetails: TFreeOTFECypher;
begin
  edDriveMountedAs.text := driveLetter+':';
  edVolLabel.text       := SDUVolumeID(driveLetter);
  edFilename.text       := '<Unknown>';
  edReadOnly.text       := '<Unknown>';
  edIVHash.text         := '<Unknown>';
  edIVCypher.text       := '<Unknown>';
  edMainCypher.text     := '<Unknown>';
  edDeviceName.text     := '<Unknown>';


  if TOTFEFreeOTFE(OTFComponent).GetVolumeInfo(driveLetter, info) then
    begin
    edFilename.text       := info.Filename;

    if info.ReadOnly then
      begin
      edReadOnly.text := 'Readonly';
      end
    else
      begin
      edReadOnly.text := 'Read/Write';
      end;


    if (
        (info.IVHashDevice = '') and
        IsEqualGUID(info.IVHashGUID, StringToGUID(NULL_GUID))
       ) then
      begin
      edIVHash.Text := 'n/a';
      end
    else
      begin
      if TOTFEFreeOTFE(OTFComponent).GetSpecificHashDetails(
                                             info.IVHashDevice,
                                             info.IVHashGUID,
                                             hashDetails
                                            ) then
        begin
        edIVHash.Text := TOTFEFreeOTFE(OTFComponent).GetHashDisplayTitle(hashDetails);
        end;
      end;



    if (
        (info.IVCypherDevice = '') and
        IsEqualGUID(info.IVCypherGUID, StringToGUID(NULL_GUID))
       ) then
      begin
      edIVCypher.Text := 'n/a';
      end
    else
      begin
      if TOTFEFreeOTFE(OTFComponent).GetSpecificCypherDetails(
                                             info.IVCypherDevice,
                                             info.IVCypherGUID,
                                             cypherDetails
                                            ) then
        begin
        edIVCypher.Text := TOTFEFreeOTFE(OTFComponent).GetCypherDisplayTitle(cypherDetails);
        end;
      end;

    if TOTFEFreeOTFE(OTFComponent).GetSpecificCypherDetails(
                                             info.MainCypherDevice,
                                             info.MainCypherGUID,
                                             cypherDetails
                                            ) then
      begin
      edMainCypher.Text := TOTFEFreeOTFE(OTFComponent).GetCypherDisplayTitle(cypherDetails);
      end;

    edDeviceName.text := info.DeviceName;
    end;

  Result := TRUE;
  
end;

END.

