unit OTFVolumeDetailsScramDisk_U;
// Description:
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  OTFVolumeDetails_U;

type
  TOTFVolumeDetailsScramDisk_F = class(TOTFVolumeDetails_F)
    GroupBox1: TGroupBox;
    Label4002: TLabel;
    Label4003: TLabel;
    Label4004: TLabel;
    Label4005: TLabel;
    Label4007: TLabel;
    Label4008: TLabel;
    Label4009: TLabel;
    reComments: TRichEdit;
    edCipherUsed: TEdit;
    edPrefDriveLetter: TEdit;
    edTimesMounted: TEdit;
    edLastAccessed: TEdit;
    edMountType: TEdit;
    edSlotNumber: TEdit;
    Label4001: TLabel;
    Label4020: TLabel;
    edReadOnly: TEdit;
    edFilename: TEdit;
    lblVolPartFilenameStr: TLabel;
    edVolLabel: TEdit;
    Label9000: TLabel;
    edDriveMountedAs: TEdit;
    Label2022: TLabel;
  private
    { Private declarations }
  public
    constructor Create(AOwner: TComponent); override;
    function  DisplayInfoForDrive(driveLetter: char): boolean; override;
  end;

implementation

{$R *.DFM}

uses
  OTFEScramDisk_U,
  SdStructures_U,
  OTFEUnified_U,
  SDUGeneral;

constructor TOTFVolumeDetailsScramDisk_F.Create(AOwner: TComponent);
begin
  inherited;

  OTFSystem:= otfesScramDisk;

end;

function TOTFVolumeDetailsScramDisk_F.DisplayInfoForDrive(driveLetter: char): boolean;
const
  notApplicable = 'n/a';
var
  slotInfo : TSlotInfo;
  mountType : string;
  ScramDisk: TOTFEScramDisk;
  slotNum: integer;
  wYear : word;
  wMonth: word;
  wDay  : word;
  OSversion: DWORD;
  RunningOnNT: boolean;
  readOnlyState: string;
begin
  ScramDisk:= TOTFEScramDisk(OTFComponent);

  OSversion := GetVersion();
  RunningOnNT := (OSversion<$80000000);

  slotNum:= ScramDisk.GetSlotForDrive(driveLetter);
  slotInfo := ScramDisk.GetSlotInfo(slotNum);

  lblVolPartFilenameStr.caption := 'Volume filename:';
  case (slotInfo.MountType) of
    mtNotMounted:
      begin
      mountType := 'Not Mounted';
      end;
    mtPartition:
      begin
      mountType := 'Partition';
      if (ScramDisk.Version()>=$300) then
        begin
        // Partition ID reporting introduced in ScramDisk v3
        lblVolPartFilenameStr.caption := 'Volume partition:';
        if slotInfo.partitionRemovable then
          begin
          mountType := mountType + ' (removable)';
          end;
        end;
      end;
    mtSVLFile:
      begin
      mountType := 'SVL file';
      end;
    mtWAVFile:
      begin
      mountType := 'WAV file';
      mountType := mountType + ' ('+inttostr(slotInfo.wavFileBits)+'/16 bits)';
      end;
    end;

  if not(RunningOnNT) then
    begin
    if ((slotInfo.bfs) and (slotInfo.MountType<>mtNotMounted)) then
      begin
      mountType := mountType + ' (bfs)';
      end;
    end;

  if ((slotInfo.viaSKFFile) and (slotInfo.MountType<>mtNotMounted)) then
    begin
    mountType := mountType + ' (via SKF)';
    end;

  edMountType.text       := mountType;
  edSlotNumber.text      := inttostr(slotInfo.SlotNo);
  edFilename.text        := notApplicable;

  if (slotInfo.MountType=mtNotMounted) then
    begin
    edDriveMountedAs.text  := notApplicable;
    edVolLabel.text        := notApplicable;
    edCipherUsed.text      := notApplicable;
    edPrefDriveLetter.text := notApplicable;
    edTimesMounted.text    := notApplicable;
    edLastAccessed.text    := notApplicable;
    edReadOnly.text        := notApplicable;
    reComments.text        := '';
    end
  else
    begin
    if  not(((slotInfo.MountType=mtPartition) and
             (ScramDisk.Version()<$300))) then
      begin
        // Partition ID reporting introduced in ScramDisk v3
      edFilename.text := slotInfo.FileName;
      end;
    edDriveMountedAs.text  := slotInfo.DriveMountedAs + ':';
    edVolLabel.text        := SDUVolumeID(slotInfo.DriveMountedAs[1]);
    edCipherUsed.text      := slotInfo.CipherType;
    edPrefDriveLetter.text := slotInfo.PreferredDrive + ':';
    edTimesMounted.text    := inttostr(slotInfo.NoAccesses);
    DecodeDate(slotInfo.LastAccess, wYear, wMonth, wDay);
    if wYear=9999 then
      begin
      edLastAccessed.text := 'Previous date/timestamp invalid';
      end
    else
      begin
      edLastAccessed.text := DateTimeToStr(slotInfo.LastAccess);
      end;
    reComments.text := slotInfo.Comments;

    readOnlyState := 'Read/Write';
    if slotInfo.readonly then
      begin
      readOnlyState := 'Readonly';
      if (slotInfo.softReadOnly and slotInfo.mediaReadOnly) then
        begin
        readOnlyState := readOnlyState+' (soft & media)';
        end
      else if slotInfo.softReadOnly then
        begin
        readOnlyState := readOnlyState+' (soft)';
        end
      else if slotInfo.mediaReadOnly then
        begin
        readOnlyState := readOnlyState+' (media)';
        end;
      end;

    edReadOnly.text := readOnlyState;
    
    end;

  Result := TRUE;

end;

END.

