unit OptionsSelfD_U;
// Description: Self-Destruct Options Dialog
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  AppGlobal_U, AppGlobalConsts_U,
  ExtCtrls,
  STUIniFile_U, Spin, Menus;

type
  TOptionsSelfD_F = class(TForm)
    pbApply: TButton;
    pbCancel: TButton;
    pbOK: TButton;
    OpenDialog: TOpenDialog;
    pcOptionsSelfD: TPageControl;
    tsTriggers: TTabSheet;
    GroupBox3: TGroupBox;
    ckEnableSelfDHotkey: TCheckBox;
    tsTargets: TTabSheet;
    tsAdditional: TTabSheet;
    GroupBox2: TGroupBox;
    edAdditionalExeFilename: TEdit;
    ckSelfDAdditionalExeEnable: TCheckBox;
    Label4: TLabel;
    lbTargetList: TListBox;
    pbUp: TButton;
    pbDown: TButton;
    pbEdit: TButton;
    pbRemove: TButton;
    ckDestroyAllMountedDrives: TCheckBox;
    Label1: TLabel;
    pbBrowseUserExecutable: TButton;
    tsEnable: TTabSheet;
    GroupBox4: TGroupBox;
    Label5: TLabel;
    Label3: TLabel;
    ckEnableSelfDestructFacilities1: TCheckBox;
    ckEnableSelfDestructFacilities2: TCheckBox;
    Label6: TLabel;
    ckQuickShredMounted: TCheckBox;
    pbAdd: TButton;
    muAddEditRemove: TPopupMenu;
    miAdd: TMenuItem;
    miEdit: TMenuItem;
    miRemove: TMenuItem;
    GroupBox5: TGroupBox;
    ckTerminateApp: TCheckBox;
    Label10: TLabel;
    tsTriggerTestMode: TTabSheet;
    GroupBox6: TGroupBox;
    Label13: TLabel;
    Label12: TLabel;
    Label11: TLabel;
    ckTriggerTestMode: TCheckBox;
    Label14: TLabel;
    ckEnableSerialPortTrigger: TCheckBox;
    ckEnableInternetTrigger: TCheckBox;
    ckEnableMenuitemTrigger: TCheckBox;
    pbConfigTriggers: TButton;
    TabSheet1: TTabSheet;
    GroupBox1: TGroupBox;
    Label2: TLabel;
    Label7: TLabel;
    ckRandomizeClock: TCheckBox;
    pnlRandomClockStart: TPanel;
    dtpRandomClockStartDate: TDateTimePicker;
    dtpRandomClockStartTime: TDateTimePicker;
    rbRandomizeClockStartCurrent: TRadioButton;
    rbUserSpecifiedStartDateTime: TRadioButton;
    pnlRandomClockEnd: TPanel;
    dtpRandomClockEndDate: TDateTimePicker;
    dtpRandomClockEndTime: TDateTimePicker;
    rbRandomizeClockEndCurrent: TRadioButton;
    rbUserSpecifiedEndDateTime: TRadioButton;
    procedure pbApplyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SetApplyButton(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
    procedure pbRemoveClick(Sender: TObject);
    procedure pbEditClick(Sender: TObject);
    procedure pbUpClick(Sender: TObject);
    procedure pbDownClick(Sender: TObject);
    procedure lbTargetListClick(Sender: TObject);
    procedure pbBrowseUserExecutableClick(Sender: TObject);
    procedure ckEnableSelfDestructFacilitiesClick(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure lbTargetListDrawItem(Control: TWinControl;
      Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure pbAddClick(Sender: TObject);
    procedure lbTargetListDblClick(Sender: TObject);
    procedure pbConfigTriggersClick(Sender: TObject);
  private
    loadingINIValues : boolean;
    procedure LoadINIValues();
    procedure EnableDisableAllControls();
    procedure AddTargetFile();
    procedure AddTargetDir();
    procedure AddTargetRegKey();
    function  EditTargetFile(var filename: string): boolean;
    function  EditTargetDir(var dirname: string): boolean;
    function  EditTargetRegKey(var regKey: string): boolean;
    procedure MoveSelectedTargets(upNotDown: boolean);
  public
  end;

var
  OptionsSelfD_F: TOptionsSelfD_F;

implementation

{$R *.DFM}

{$IFDEF LINUX}
This software is only intended for use under MS Windows
{$ENDIF}
{$WARN UNIT_PLATFORM OFF}  // Useless warning about platform - we're already
                           // protecting against that!

uses
  FileCtrl,
  RegKeyEntry_U, TargetTypeSelect_U,
  TriggerConfig_U, SDUGeneral;

procedure TOptionsSelfD_F.pbApplyClick(Sender: TObject);
begin
  // If either of the enable checkboxes isn't checked, uncheck both of
  // them
  if (not(ckEnableSelfDestructFacilities1.checked) or
      not(ckEnableSelfDestructFacilities2.checked)) then
    begin
    ckEnableSelfDestructFacilities1.checked := FALSE;
    ckEnableSelfDestructFacilities2.checked := FALSE;
    end;

  // Enable page...
  STU_INI_FILE.OptSelfDFacilitiesEnable1 := ckEnableSelfDestructFacilities1.checked;
  STU_INI_FILE.OptSelfDFacilitiesEnable2 := ckEnableSelfDestructFacilities2.checked;

  // Triggers page...
  STU_INI_FILE.OptSelfDEnableHotkey := ckEnableSelfDHotkey.checked;
  STU_INI_FILE.OptSelfDEnableMenuitem := ckEnableMenuitemTrigger.checked;
  STU_INI_FILE.OptSelfDTriggerPortEnable := ckEnableSerialPortTrigger.checked;
  STU_INI_FILE.OptSelfDEnableInternetTrigger := ckEnableInternetTrigger.checked;

  // Tigger test mode...
  STU_INI_FILE.OptSelfDTriggerTestMode := ckTriggerTestMode.checked;

  // Self-destruct Targets
  STU_INI_FILE.OptSelfDDestroyMountedEnable := ckDestroyAllMountedDrives.checked;
  STU_INI_FILE.OptSelfDDestroyMountedQuickShred := ckQuickShredMounted.checked;
  STU_INI_FILE.OptSelfDTargetFiles.assign(lbTargetList.items);

  // Self-destruct Additional
  STU_INI_FILE.OptSelfDRunAdditionalEnable := ckSelfDAdditionalExeEnable.checked;
  STU_INI_FILE.OptSelfDRunAdditionalFilename := edAdditionalExeFilename.text;
  STU_INI_FILE.OptSelfDStealth := ckTerminateApp.checked;

  STU_INI_FILE.OptSelfDRandomizeDT := ckRandomizeClock.checked;
  dtpRandomClockStartDate.Time := dtpRandomClockStartTime.Time;
  STU_INI_FILE.OptSelfDRandomizeDTStart := dtpRandomClockStartDate.DateTime;
  dtpRandomClockEndDate.Time := dtpRandomClockEndTime.Time;
  STU_INI_FILE.OptSelfDRandomizeDTEnd := dtpRandomClockEndDate.DateTime;
  STU_INI_FILE.OptSelfDRandomizeDTStartCurrent := rbRandomizeClockStartCurrent.checked;
  STU_INI_FILE.OptSelfDRandomizeDTEndCurrent := rbRandomizeClockEndCurrent.checked;


  if (STU_INI_FILE.OptSelfDRandomizeDT) and
     (
      // Start time specified is later than end time specified
      (STU_INI_FILE.OptSelfDRandomizeDTStart>STU_INI_FILE.OptSelfDRandomizeDTEnd)

      or

      // End time earlier than current time
      ((STU_INI_FILE.OptSelfDRandomizeDTStartCurrent) and
       (STU_INI_FILE.OptSelfDRandomizeDTEnd<now))

      or

      // Start time later than current time
      ((STU_INI_FILE.OptSelfDRandomizeDTEndCurrent) and
       (STU_INI_FILE.OptSelfDRandomizeDTStart>now))
     ) then
    begin
    STU_INI_FILE.OptSelfDRandomizeDT := FALSE;
    showmessage('For randomizing the system clock, the start date/time specified must occur before the end date/time'+CRLF+CRLF+
                'System clock randomizing on self-destruct is disabled until this is corrected');
    end;

  // Finish off...
  STU_INI_FILE.WriteIniFile;
  pbApply.enabled := FALSE;

  LoadINIValues();
end;

procedure TOptionsSelfD_F.FormCreate(Sender: TObject);
begin
  pnlRandomClockStart.bevelouter := bvNone;
  pnlRandomClockEnd.bevelouter := bvNone;
  
  LoadINIValues();
  pcOptionsSelfD.activepage := tsEnable;
  EnableDisableAllControls();

end;

procedure TOptionsSelfD_F.LoadINIValues();
begin
  loadingINIValues := TRUE;
  STU_INI_FILE.ReadIniFile;

  // Self-destruct page...
  ckEnableSelfDestructFacilities1.checked := STU_INI_FILE.OptSelfDFacilitiesEnable1;
  ckEnableSelfDestructFacilities2.checked := STU_INI_FILE.OptSelfDFacilitiesEnable2;

  // Triggers page...
  ckEnableSelfDHotkey.checked := STU_INI_FILE.OptSelfDEnableHotkey;
  ckEnableMenuitemTrigger.checked := STU_INI_FILE.OptSelfDEnableMenuitem;
  ckEnableSerialPortTrigger.checked := STU_INI_FILE.OptSelfDTriggerPortEnable;
  ckEnableInternetTrigger.checked := STU_INI_FILE.OptSelfDEnableInternetTrigger;

  // Test mode page...
  ckTriggerTestMode.checked := STU_INI_FILE.OptSelfDTriggerTestMode;

  // Self-destruct Targets page...
  ckDestroyAllMountedDrives.checked := STU_INI_FILE.OptSelfDDestroyMountedEnable;
  ckQuickShredMounted.checked := STU_INI_FILE.OptSelfDDestroyMountedQuickShred;
  lbTargetList.items.assign(STU_INI_FILE.OptSelfDTargetFiles);

  // System clock page...
  ckRandomizeClock.checked := STU_INI_FILE.OptSelfDRandomizeDT;
  dtpRandomClockStartDate.DateTime := STU_INI_FILE.OptSelfDRandomizeDTStart;
  dtpRandomClockStartTime.DateTime := STU_INI_FILE.OptSelfDRandomizeDTStart;
  dtpRandomClockEndDate.DateTime := STU_INI_FILE.OptSelfDRandomizeDTEnd;
  dtpRandomClockEndTime.DateTime := STU_INI_FILE.OptSelfDRandomizeDTEnd;
  rbRandomizeClockStartCurrent.checked := STU_INI_FILE.OptSelfDRandomizeDTStartCurrent;
  rbUserSpecifiedStartDateTime.checked := not(rbRandomizeClockStartCurrent.checked);
  rbRandomizeClockEndCurrent.checked := STU_INI_FILE.OptSelfDRandomizeDTEndCurrent;
  rbUserSpecifiedEndDateTime.checked := not(rbRandomizeClockEndCurrent.checked);

  // Self-destruct Additional page...
  ckSelfDAdditionalExeEnable.checked := STU_INI_FILE.OptSelfDRunAdditionalEnable;
  edAdditionalExeFilename.text := STU_INI_FILE.OptSelfDRunAdditionalFilename;
  ckTerminateApp.checked := STU_INI_FILE.OptSelfDStealth;

  // Finish off...
  pbApply.enabled := FALSE;
  EnableDisableAllControls();
  loadingINIValues := FALSE;
end;

procedure TOptionsSelfD_F.SetApplyButton(Sender: TObject);
begin
  EnableDisableAllControls();
  pbApply.enabled := TRUE;

end;

procedure TOptionsSelfD_F.pbCancelClick(Sender: TObject);
begin
  // Reset everything...
  LoadINIValues;

  modalresult := mrCancel;

end;

procedure TOptionsSelfD_F.pbRemoveClick(Sender: TObject);
var
  i: integer;
  confirm: Word;
begin
  confirm := messageDlg('Are you sure you wish to remove the selected targets?', mtConfirmation, [mbYes, mbNo], 0);
  if confirm=mrYes then
    begin
    for i:=(lbTargetList.items.count-1) downto 0 do
      begin
      if lbTargetList.Selected[i] then
        begin
        lbTargetList.items.delete(i);
        end;
      end;

    EnableDisableAllControls();
    SetApplyButton(nil);
  end;

end;

procedure TOptionsSelfD_F.pbEditClick(Sender: TObject);
var
  currentTarget: string;
begin
  currentTarget := lbTargetList.items[lbTargetList.ItemIndex];
  if Pos('DIR:', currentTarget)=1 then
    begin
    Delete(currentTarget, 1, 4);
    if EditTargetDir(currentTarget) then
      begin
      lbTargetList.items[lbTargetList.ItemIndex] := 'DIR:'+currentTarget;
      SetApplyButton(nil);
      end;
    end
  else if Pos('FILE:', currentTarget)=1 then
    begin
    Delete(currentTarget, 1, 5);
    if EditTargetFile(currentTarget) then
      begin
      lbTargetList.items[lbTargetList.ItemIndex] := 'FILE:'+currentTarget;
      SetApplyButton(nil);
      end;
    end
  else if Pos('REGKEY:', currentTarget)=1 then
    begin
    Delete(currentTarget, 1, 7);
    if EditTargetRegKey(currentTarget) then
      begin
      lbTargetList.items[lbTargetList.ItemIndex] := 'REGKEY:'+currentTarget;
      SetApplyButton(nil);
      end;
    end;

  EnableDisableAllControls();

end;

procedure TOptionsSelfD_F.pbUpClick(Sender: TObject);
begin
  MoveSelectedTargets(TRUE);

end;

procedure TOptionsSelfD_F.pbDownClick(
  Sender: TObject);
begin
  MoveSelectedTargets(FALSE);

end;


procedure TOptionsSelfD_F.EnableDisableAllControls();
begin
  SDUEnableControl(ckQuickShredMounted, ckDestroyAllMountedDrives.checked);
  SDUEnableControl(edAdditionalExeFilename, ckSelfDAdditionalExeEnable.checked);
  SDUEnableControl(pbBrowseUserExecutable, ckSelfDAdditionalExeEnable.checked);

  rbRandomizeClockStartCurrent.enabled := ckRandomizeClock.checked;
  rbUserSpecifiedStartDateTime.enabled := ckRandomizeClock.checked;
  rbRandomizeClockEndCurrent.enabled := ckRandomizeClock.checked;
  rbUserSpecifiedEndDateTime.enabled   := ckRandomizeClock.checked;
  SDUEnableControl(dtpRandomClockStartDate, (ckRandomizeClock.checked and not(rbRandomizeClockStartCurrent.checked)));
  SDUEnableControl(dtpRandomClockStartTime, (ckRandomizeClock.checked and not(rbRandomizeClockStartCurrent.checked)));
  SDUEnableControl(dtpRandomClockEndDate, (ckRandomizeClock.checked and not(rbRandomizeClockEndCurrent.checked)));
  SDUEnableControl(dtpRandomClockEndTime, (ckRandomizeClock.checked and not(rbRandomizeClockEndCurrent.checked)));

  if lbTargetList.SelCount=1 then
    begin
    pbEdit.enabled := (lbTargetList.Items.Count>0);
    end
  else
    begin
    pbEdit.enabled := FALSE;
    end;

  if (lbTargetList.SelCount>=1) AND (lbTargetList.Items.Count>0) then
    begin
    pbRemove.enabled := TRUE;

    pbUp.enabled := lbTargetList.Items.Count>0;
    pbDown.enabled := lbTargetList.Items.Count>0;

    if lbTargetList.Selected[0] then
      begin
      pbUp.enabled := FALSE;
      end;

    if lbTargetList.Selected[(lbTargetList.Items.Count-1)] then
      begin
      pbDown.enabled := FALSE;
      end;

    end
  else
    begin
    pbRemove.enabled := FALSE;
    pbUp.enabled := FALSE;
    pbDown.enabled := FALSE;
    end;

  miAdd.enabled := pbAdd.enabled;
  miEdit.enabled := pbEdit.enabled;
  miRemove.enabled := pbRemove.enabled;
end;

procedure TOptionsSelfD_F.lbTargetListClick(Sender: TObject);
begin
  EnableDisableAllControls();

end;

procedure TOptionsSelfD_F.pbBrowseUserExecutableClick(Sender: TObject);
begin
  OpenDialog.title := 'Select Executable';
  OpenDialog.filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edAdditionalExeFilename.text);
  if OpenDialog.execute then
    begin
    edAdditionalExeFilename.text := OpenDialog.filename;
    end;

end;

procedure TOptionsSelfD_F.ckEnableSelfDestructFacilitiesClick(
  Sender: TObject);
var
  confirm : word;
  applyBeforeRevertCkBox : boolean;
begin
  // Check to make sure that the tsSelfDestruct page is active, otherwise
  // it could just be initialising
  if not(loadingINIValues) then
    begin
    if ((ckEnableSelfDestructFacilities1.checked) AND
        (ckEnableSelfDestructFacilities2.checked)) then
      begin
      confirm := messageDlg('You have opted to ENABLE the SELF-DESTRUCT facilities. Are you certain that you wish to do this?',
                 mtWarning,
                 [mbYes, mbNo],
                 0);

      if confirm=mrYes then
        begin
        SetApplyButton(nil);
        end
      else
        begin
        // We need to save the state of the Apply button and restore it
        // because when the checkbox is unchecked, this routine is called
        // again
        applyBeforeRevertCkBox := pbApply.enabled;
        ckEnableSelfDestructFacilities1.checked := FALSE;
        ckEnableSelfDestructFacilities2.checked := FALSE;
        pbApply.enabled := applyBeforeRevertCkBox;
        end;

      end // User has tried to ENABLE the self-destruct hotkey
    else
      begin
      SetApplyButton(nil);
      end; // User has DISABLED the self-destruct hotkey

    end; // The hotkeys page is active

end;

procedure TOptionsSelfD_F.pbOKClick(Sender: TObject);
begin
  if pbApply.enabled then
    begin
    pbApplyClick(sender);
    end;

  modalresult := mrOK;

end;

procedure TOptionsSelfD_F.lbTargetListDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Bitmap: TBitmap;      { temporary variable for the items bitmap }
  Offset: Integer;      { text offset width }
  pictureRect: TRect;
  itemText: string;
begin
  lbTargetList.Canvas.FillRect(Rect);       { clear the rectangle }
  Offset := 2;          { provide default offset }

  Bitmap := TBitmap.create();
  Bitmap.Transparent := TRUE;
  Bitmap.ReleaseHandle;

  itemText := lbTargetList.Items[Index];

  if Pos('DIR:', lbTargetList.Items[Index])=1 then
    begin
    Bitmap.Handle := LoadBitmap(hInstance, 'BITMAP_DIR');
    Delete(itemText, 1, 4);
    end
  else if Pos('FILE:', lbTargetList.Items[Index])=1 then
    begin
    Bitmap.Handle := LoadBitmap(hInstance, 'BITMAP_FILE');
    Delete(itemText, 1, 5);
    end
  else if Pos('REGKEY:', lbTargetList.Items[Index])=1 then
    begin
    Bitmap.Handle := LoadBitmap(hInstance, 'BITMAP_REGKEY');
    Delete(itemText, 1, 7);
    end
  else
    begin
    Bitmap.Handle := LoadBitmap(hInstance, 'BITMAP_SMALL_BLANK');
    end;

  if Bitmap <> nil then
    begin
    pictureRect.top := pictureRect.top + ((lbTargetList.ItemHeight-Bitmap.height) div 2);
    pictureRect := Rect;
    pictureRect.top := pictureRect.top+2;
    pictureRect.left := pictureRect.left+2;
    lbTargetList.Canvas.Draw(pictureRect.left, pictureRect.top, Bitmap);
    Offset := Bitmap.width + 6;    { add four pixels between bitmap and text}
    end;

  lbTargetList.canvas.TextOut(Rect.Left + Offset, Rect.Top, itemText)  { display the text }

end;

procedure TOptionsSelfD_F.pbAddClick(Sender: TObject);
var
  selectTypeDlg: TTargetTypeSelect_F;
begin
  selectTypeDlg := TTargetTypeSelect_F.create(nil);
  try
    if selectTypeDlg.showmodal()=mrOK then
      begin
      case selectTypeDlg.rgTargetType.ItemIndex of
        0: AddTargetFile();
        1: AddTargetDir();
        2: AddTargetRegKey();
        end;
      end;
  finally
    selectTypeDlg.Free();
  end;

end;

procedure TOptionsSelfD_F.AddTargetFile();
begin
  OpenDialog.title := 'Add Target File';
  OpenDialog.filter := FILTER_ALLFILES;
  OpenDialog.filename := '';
  if OpenDialog.execute then
    begin
    lbTargetList.items.add('FILE:'+OpenDialog.filename);
    EnableDisableAllControls();
    SetApplyButton(nil);
    end;
end;

procedure TOptionsSelfD_F.AddTargetDir();
var
  dirToShred: string;
begin
  if SelectDirectory('Select Directory to Shred', '', dirToShred) then
    begin
    lbTargetList.items.add('DIR:'+dirToShred);
    EnableDisableAllControls();
    SetApplyButton(nil);
    end;

end;

procedure TOptionsSelfD_F.AddTargetRegKey();
var
  regKeyDlg: TRegKeyEntry_F;
begin
  regKeyDlg := TRegKeyEntry_F.create(nil);
  try
    if regKeyDlg.ShowModal()=mrOK then
      begin
      lbTargetList.items.add('REGKEY:'+regKeyDlg.RegistryKey);
      EnableDisableAllControls();
      SetApplyButton(nil);
      end;
  finally
    regKeyDlg.Free();
  end;
end;


function TOptionsSelfD_F.EditTargetFile(var filename: string): boolean;
begin
  Result := FALSE;
  OpenDialog.title := 'Edit Target File';
  OpenDialog.filter := FILTER_ALLFILES;
  OpenDialog.filename:=filename;
  if OpenDialog.execute then
    begin
    filename := OpenDialog.filename;
    Result := TRUE;
    end;

end;


function TOptionsSelfD_F.EditTargetDir(var dirname: string): boolean;
var
  dirToShred: string;
begin
  Result := FALSE;
  dirToShred := dirname;
  // This should use dirname to set the default directory, but this doesn't
  //appear to be able to do that. Using "SelectDirectory(dirToShred, [], 0)"
  // should work, but this dialog looks naff, and it also changed the current
  // directory (which it shouldn't, according to the delphi help file)
  if SelectDirectory('Select Directory to Shred', '', dirToShred) then
    begin
    dirname := dirToShred;
    Result := TRUE;
    end;
end;


function TOptionsSelfD_F.EditTargetRegKey(var regKey: string): boolean;
var
  regKeyDlg: TRegKeyEntry_F;
begin
  Result := FALSE;
  regKeyDlg := TRegKeyEntry_F.create(nil);
  try
    regKeyDlg.RegistryKey := regKey;
    if regKeyDlg.ShowModal()=mrOK then
      begin
      regKey := regKeyDlg.RegistryKey;
      Result := TRUE;
      end;
  finally
    regKeyDlg.Free();
  end;
end;


// Move all selected items in the listbox up or down
procedure TOptionsSelfD_F.MoveSelectedTargets(upNotDown: boolean);
var
  i: integer;
  finish: integer;
  tempString : string;
  directionInt: integer;
  tempSelected: boolean;
begin
  if upNotDown then
    begin
    directionInt := -1;
    i := 0;
    finish := (lbTargetList.items.count-1);
    end
  else
    begin
    directionInt := 1;
    i := (lbTargetList.items.count-1);
    finish := 0;
    end;

  while i<>finish do
    begin
    i := i - directionInt;
    if lbTargetList.Selected[i] then
      begin
      tempString := lbTargetList.items[i+directionInt];
      tempSelected := lbTargetList.Selected[i+directionInt];
      lbTargetList.items[i+directionInt] := lbTargetList.items[i];
      lbTargetList.Selected[i+directionInt] := TRUE;
      lbTargetList.items[i] := tempString;
      lbTargetList.Selected[i] := tempSelected;
      end;
    end;
    
  EnableDisableAllControls();

  SetApplyButton(nil);
end;

procedure TOptionsSelfD_F.lbTargetListDblClick(Sender: TObject);
begin
  if pbEdit.enabled then
    begin
    pbEditClick(nil);
    end;
end;

procedure TOptionsSelfD_F.pbConfigTriggersClick(Sender: TObject);
var
  trigConfigDlg: TTriggerConfig_F;
begin
  trigConfigDlg:= TTriggerConfig_F.Create(nil);
  try
    trigConfigDlg.showmodal();
  finally
    trigConfigDlg.Free();
  end;
end;


// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
{$WARN UNIT_PLATFORM ON}
// -----------------------------------------------------------------------------

END.

