unit ShortCut;

interface

uses ShlObj, ActiveX, ComObj, Windows,
     AppGlobal_U,
     WinInfo;

// TARGET is the file that the shortcut will point to. 

// ARGS is the command-line arguments you wish to pass to the Target app. 

// WORKDIR is the directory that will be current when the link is launched. 

// SHORTCUTNAME is the name of the shortcut link file. 

// LOC is the location the file should go. Uses the WinInfo unit to get the location 
//   of special system locations. Set to slNone if ShortCutName fully defines the 
//   location of the shortcut. 

type 
  TSpecialLocations = (slNone,slDesktop,slFavorites,slFonts,slNetHood,slPersonal, 
                       slPrograms,slRecent,slSendTo,slStartMenu,slStartup,slTemplates); 

procedure CreateLink(Target,Args,WorkDir,ShortCutName: String; Loc: TSpecialLocations);
function  CheckLink(ShortCutName: String; Loc: TSpecialLocations): boolean;
procedure DeleteLink(ShortCutName: String; Loc: TSpecialLocations);
function GetShortCutLocation(ShortCutName: String; Loc: TSpecialLocations): string;

implementation

uses
  SysUtils;

procedure CreateLink(Target,Args,WorkDir,ShortCutName: String; Loc: TSpecialLocations); 
var
  IObj    : IUnknown; 
  Link    : IShellLink; 
  IPFile  : IPersistFile; 
  TargetW : WideString;
begin
  IObj := CreateComObject(CLSID_ShellLink);
  Link := IObj as IShellLink;
  IPFile  := IObj as IPersistFile;
  with Link do
    begin
      SetPath(PChar(Target));
      SetArguments(PChar(Args));
      SetWorkingDirectory(PChar(WorkDir));
    end;

  TargetW := GetShortCutLocation(ShortCutName, Loc);
  IPFile.Save(PWChar(TargetW),False);
end;

function CheckLink(ShortCutName: String; Loc: TSpecialLocations): boolean;
var
  target: string;
begin
  target := GetShortCutLocation(ShortCutName, Loc);
  Result := FileExists(target);
end;

procedure DeleteLink(ShortCutName: String; Loc: TSpecialLocations);
var
  target: string;
begin
  target := GetShortCutLocation(ShortCutName, Loc);
  DeleteFile(target);
end;

function GetShortCutLocation(ShortCutName: String; Loc: TSpecialLocations): string;
var
  TargetW: string;
begin
  case Loc of
    slDesktop   : TargetW := WindowsInfo.DesktopDir+'\'+ShortCutName;
    slFavorites : TargetW := WindowsInfo.FavoritesDir+'\'+ShortCutName;
    slFonts     : TargetW := WindowsInfo.FontsDir+'\'+ShortCutName;
    slNetHood   : TargetW := WindowsInfo.NetHoodDir+'\'+ShortCutName;
    slPersonal  : TargetW := WindowsInfo.PersonalDir+'\'+ShortCutName;
    slPrograms  : TargetW := WindowsInfo.ProgramsDir+'\'+ShortCutName;
    slRecent    : TargetW := WindowsInfo.RecentDir+'\'+ShortCutName;
    slSendTo    : TargetW := WindowsInfo.SendToDir+'\'+ShortCutName;
    slStartMenu : TargetW := WindowsInfo.StartMenuDir+'\'+ShortCutName;
    slStartup   : TargetW := WindowsInfo.StartupDir+'\'+ShortCutName;
    slTemplates : TargetW := WindowsInfo.TemplatesDir+'\'+ShortCutName;
  else            TargetW := ShortCutName;
  end;

  Result := TargetW;
end;

END.


