unit TriggerConfig_U;
// Description: Self-Destruct Trigger Configuration Dialog
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Spin, ExtCtrls;

type
  TTriggerConfig_F = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    pbApply: TButton;
    pcTriggerConfig: TPageControl;
    tsInternet: TTabSheet;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    seNumChalResp: TSpinEdit;
    sePortNum: TSpinEdit;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    edChal: TEdit;
    edResp: TEdit;
    seEditChalRespNum: TSpinEdit;
    pbSetChalResp: TButton;
    tsHotkeyMenuitem: TTabSheet;
    tsSerial: TTabSheet;
    tsModem: TTabSheet;
    GroupBox3: TGroupBox;
    lblSelfDestruct: TLabel;
    hkSelfDHotkey: THotKey;
    GroupBox4: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    cbPorts: TComboBox;
    seSerialPollInterval: TSpinEdit;
    pbPortTest: TButton;
    Label12: TLabel;
    rgSelfDMenuitem: TRadioGroup;
    procedure seNumChalRespChange(Sender: TObject);
    procedure seEditChalRespNumChange(Sender: TObject);
    procedure pbSetChalRespClick(Sender: TObject);
    procedure pbPortTestClick(Sender: TObject);
    procedure pbApplyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SetApplyButton(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
  private
    loadingINIValues : boolean;
    procedure LoadINIValues();
    procedure GetCOMPorts();
  public
    { Public declarations }
  end;

var
  CHALL: array [1..5] of string;
  RESPE: array [1..5] of string;

implementation

{$R *.DFM}

uses AppGlobal_U,
     STUINIFile_U,
     Menus, // Menus needed for ShortCutToText and TextToShortCut
     COMPort_U,
     SDUGeneral;

procedure TTriggerConfig_F.seNumChalRespChange(Sender: TObject);
begin
  setlength(STU_INI_FILE.OptSelfDInternetChal, seNumChalResp.value);
  setlength(STU_INI_FILE.OptSelfDInternetResp, seNumChalResp.value);

  seEditChalRespNum.MaxValue := seNumChalResp.value;
  if seEditChalRespNum.value>seEditChalRespNum.MaxValue then
    begin
    seEditChalRespNum.value := seEditChalRespNum.MaxValue;
    end;

  SetApplyButton(nil);
end;

procedure TTriggerConfig_F.seEditChalRespNumChange(Sender: TObject);
begin
  if (seEditChalRespNum.value>0) AND
     (seEditChalRespNum.value<=seEditChalRespNum.maxvalue) then
    begin
    edChal.text := STU_INI_FILE.OptSelfDInternetChal[seEditChalRespNum.value-1];
    edResp.text := STU_INI_FILE.OptSelfDInternetResp[seEditChalRespNum.value-1];
    end;

  if (seEditChalRespNum.value>seEditChalRespNum.maxvalue) then
    begin
    seEditChalRespNum.value:= seEditChalRespNum.maxvalue;
    end
  else if (seEditChalRespNum.value<1) then
    begin
    seEditChalRespNum.value:= 1;
    end;

end;

procedure TTriggerConfig_F.pbSetChalRespClick(Sender: TObject);
begin
  STU_INI_FILE.OptSelfDInternetChal[seEditChalRespNum.value-1] := edChal.text;
  STU_INI_FILE.OptSelfDInternetResp[seEditChalRespNum.value-1] := edResp.text;

  SetApplyButton(nil);
end;

procedure TTriggerConfig_F.pbPortTestClick(Sender: TObject);
begin
  if (WinExec('PortTest.exe', SW_RESTORE))<31 then
    begin
    ErrorMessage('Error running PortTest.exe');
    end;

end;

procedure TTriggerConfig_F.pbApplyClick(Sender: TObject);
begin
  case rgSelfDMenuitem.itemindex of
    0 : STU_INI_FILE.OptSelfDMenuitemLocation := sdmiTop;
    1 : STU_INI_FILE.OptSelfDMenuitemLocation := sdmiBottom;
  else
    STU_INI_FILE.OptSelfDMenuitemLocation := sdmiBoth;
  end;

  STU_INI_FILE.OptSelfDHotkey := ShortCutToText(hkSelfDHotkey.hotkey);
                                     
  STU_INI_FILE.OptSelfDTriggerPort := cbPorts.text;
  STU_INI_FILE.OptSelfDTriggerPortInterval := seSerialPollInterval.value;

  STU_INI_FILE.OptSelfDInternetPort := sePortNum.value;
  STU_INI_FILE.OptSelfDInternetCRPairs := seNumChalResp.value;

  // Finish off...
  STU_INI_FILE.WriteIniFile;
  pbApply.enabled := FALSE;

  LoadINIValues();
end;

procedure TTriggerConfig_F.LoadINIValues();
var
  i: integer;
  portLocatedInList: boolean;
begin
  loadingINIValues := TRUE;
  STU_INI_FILE.ReadIniFile;

  hkSelfDHotkey.hotkey := TextToShortCut(STU_INI_FILE.OptSelfDHotkey);

  case STU_INI_FILE.OptSelfDMenuitemLocation of
    sdmiTop    : rgSelfDMenuitem.itemindex := 0;
    sdmiBottom : rgSelfDMenuitem.itemindex := 1;
  else
    rgSelfDMenuitem.itemindex := 2;
  end;


  portLocatedInList := FALSE;
  // while loop needed in case the port isn't located the 1st time round, and
  // it gets inserted; it should be found the 2nd time round
  while not(portLocatedInList) do
    begin
    for i:=0 to (cbPorts.items.count-1) do
      begin
      if cbPorts.items[i]=STU_INI_FILE.OptSelfDTriggerPort then
        begin
        cbPorts.itemindex := i;
        portLocatedInList := TRUE;
        end;
      end;

      // if portLocatedInList is FALSE then presumably it's not in the list the
      // port was already opened (probably by us!). Tack the port on to the list
      if STU_INI_FILE.OptSelfDTriggerPort='' then
        begin
        portLocatedInList := TRUE
        end;
        
      if not(portLocatedInList) then
        begin
        cbPorts.Sorted := TRUE;
        cbPorts.Sorted := FALSE;
        cbPorts.Sorted := TRUE;
        cbPorts.items.add(STU_INI_FILE.OptSelfDTriggerPort);
        cbPorts.Sorted := FALSE;
        end;
    end; // while not(portLocatedInList)
  seSerialPollInterval.value := STU_INI_FILE.OptSelfDTriggerPortInterval;

  sePortNum.value := STU_INI_FILE.OptSelfDInternetPort;
  seNumChalResp.value := STU_INI_FILE.OptSelfDInternetCRPairs;
  seNumChalRespChange(nil);
  seEditChalRespNumChange(nil);

  // Finish off...
  pbApply.enabled := FALSE;
  loadingINIValues := FALSE;
end;

procedure TTriggerConfig_F.FormCreate(Sender: TObject);
begin
  loadingINIValues := TRUE;
  GetCOMPorts();
  loadingINIValues := FALSE;

  LoadINIValues();
  pcTriggerConfig.activepage := tsHotkeyMenuitem;

end;

procedure TTriggerConfig_F.GetCOMPorts();
var
  portObj: TCOMPort;
begin
  portObj:= TCOMPort.create();
  try
    portObj.GetPorts(TStringList(cbPorts.items));
    cbPorts.itemindex:=0;
  finally
    portObj.Free();
  end;
end;



procedure TTriggerConfig_F.SetApplyButton(Sender: TObject);
begin
  pbApply.enabled := TRUE;

end;

procedure TTriggerConfig_F.pbOKClick(Sender: TObject);
begin
  if pbApply.enabled then
    begin
    pbApplyClick(sender);
    end;

  modalresult := mrOK;

end;

procedure TTriggerConfig_F.pbCancelClick(Sender: TObject);
begin
  LoadINIValues();
  ModalResult := mrCancel;

end;

END.

