unit VolumeInfo_U;
// Description:
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//

// !!!!!!!!!!!!!
// !! WARNING !!
// !!!!!!!!!!!!!
// TheMainForm must be set to a TMain_F as this dialog may call
//    TheMainForm.OTFCryptoMountPrompted();
//    TheMainForm.OTFCryptoMountUnPrompted();
//    TheMainForm.OTFCryptoMountDevices();
//    TheMainForm.OTFCryptoDismountAll(emergency)
// to mount volumes

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids,
  Menus, ComCtrls, ExtCtrls,
  OTFEUnified_U,
  OTFVolumeDetails_U,
  Main_U,
  ImgList;

const
  VOL_INF_DLG_COL_DRIVE       = 'Drive';
  VOL_INF_DLG_COL_VOLLABEL    = 'Volume label';
  VOL_INF_DLG_COL_VOLFILENAME = 'Volume filename';

type
  TVolumeInfo_F = class(TForm)
    muDriveListContextMenu: TPopupMenu;
    miExplore: TMenuItem;
    miLinePopupLine0: TMenuItem;
    miMountFile2: TMenuItem;
    miLinePopupLine1: TMenuItem;
    miDismountNormal2: TMenuItem;
    miDismountEmergency2: TMenuItem;
    miLinePopupLine2: TMenuItem;
    miRefresh: TMenuItem;
    muMainMenu: TMainMenu;
    miVolume: TMenuItem;
    miMountFile: TMenuItem;
    miMountPartition: TMenuItem;
    miLineVolume1: TMenuItem;
    miDismountNormal: TMenuItem;
    miDismountEmergency: TMenuItem;
    miLineVolume3: TMenuItem;
    miClose: TMenuItem;
    miExplore2: TMenuItem;
    miProperties: TMenuItem;
    N5: TMenuItem;
    miProperties2: TMenuItem;
    miMountPartition2: TMenuItem;
    Refresh1: TMenuItem;
    ilDriveTypes: TImageList;
    miView: TMenuItem;
    N6: TMenuItem;
    N7: TMenuItem;
    pnlDrivesListPlaceHolder: TPanel;
    sgrMountedSummary: TStringGrid;
    tvMountedSummary: TTreeView;
    StatusBar: TStatusBar;
    miLineVolume2: TMenuItem;
    miDismountAllNormal: TMenuItem;
    miDismountAllEmergency: TMenuItem;
    miDisplayNonEncrypted: TMenuItem;
    miDisplayVolInfo: TMenuItem;
    pnlInfoSpacing: TPanel;
    pnlInfoPlaceHolder: TPanel;
    miExit: TMenuItem;
    lvDrives: TListView;
    ilDriveIcons: TImageList;
    miViewAsIcons: TMenuItem;
    miViewAsTree: TMenuItem;
    miViewAsTable: TMenuItem;
    N1: TMenuItem;

    procedure WMDeviceChange(var Msg : TMessage); message WM_DEVICECHANGE;
    procedure miRefreshClick(Sender: TObject);
    procedure sgrMountedSummaryClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure miExploreClick(Sender: TObject);
    procedure pnlInfoPlaceHolderResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sgrMountedSummaryKeyPress(Sender: TObject; var Key: Char);
    procedure sgrMountedSummaryColumnMoved(Sender: TObject; FromIndex,
      ToIndex: Integer);
    procedure miPropertiesClick(Sender: TObject);
    procedure miCloseClick(Sender: TObject);
    procedure miDismountNormalClick(Sender: TObject);
    procedure miDismountEmergencyClick(Sender: TObject);
    procedure miDismountAllNormalClick(Sender: TObject);
    procedure miDismountAllEmergencyClick(Sender: TObject);
    procedure ListSummaryKeyPress(Sender: TObject; var Key: Char);
    procedure tvMountedSummaryChange(Sender: TObject; Node: TTreeNode);
    procedure miMountFileClick(Sender: TObject);
    procedure miMountPartitionClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure miDisplayVolInfoClick(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ViewChangeClick(Sender: TObject);
    procedure miDisplayNonEncryptedClick(Sender: TObject);
    procedure lvDrivesChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure lvDrivesClick(Sender: TObject);
  private
    VolumeInfoForms: array [TOTFESystem] of TOTFVolumeDetails_F;
    VolumeSelected: boolean;
    VolumeSelectedType: TOTFESystem;
    OTFDrives: string;
    displayBeingRefreshed: boolean;
    MountingFilesEnabled: boolean;

    function  AttemptSelectDrive(drive: char): boolean;

    procedure WMSysCommand(var msg: TWMSysCommand); message WM_SYSCOMMAND;
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;

    procedure EnableDisableControls();
    procedure DismountAll(emergency: boolean);
    procedure DisplayVolumeInfo(drive: char);
    function  DriveSelected(): char;
    function  ReportOTFVolumes(): string;
    procedure SaveSettings();
    procedure SetDisplayVolInfoPanel(display: boolean);
    procedure RefreshVolInfoPanelSize();
    procedure BlankVolInfoDisplay();
    procedure ClearDownTreeViewAndImages();

    procedure InitIconsForDrives();
    function  AddIconForDrive(driveLetter: char): integer;

  public
    OTFEUnified: TOTFEUnified;
    TheMainForm: TMain_F;
    procedure InitDlg();
    procedure RefreshDisplay();
    procedure SelectDrive(drive: char);
    procedure SelectFirstMtdDrive();
  end;


implementation

{$R *.DFM}

{$IFDEF LINUX}
This software is only intended for use under MS Windows
{$ENDIF}
{$WARN UNIT_PLATFORM OFF}  // Useless warning about platform - we're already
                           // protecting against that!

uses
  ShellApi,
  AppGlobal_U,
  SDUGeneral,
  OTFVolumeDetailsFreeOTFE_U,
  OTFVolumeDetailsBestCrypt_U,
  OTFVolumeDetailsE4M_U,
  OTFVolumeDetailsTrueCrypt_U,
  OTFVolumeDetailsPGPDisk_U,
  OTFVolumeDetailsScramDisk_U,
  OTFVolumeDetailsCrossCrypt_U,
  FileCtrl,
  Commctrl, // required for ImageList_GetIcon
  shlObj; // required for PITEMIDLIST


procedure TVolumeInfo_F.SelectDrive(drive: char);
var
  found: boolean;
begin
  found := FALSE;

  if drive<>#0 then
    begin
    found := AttemptSelectDrive(drive);
    end;

  if not(found) then
    begin
    SelectFirstMtdDrive();
    end;

end;


// This function is required to to prevent looping between SelectDrive and
// SelectFirstMtdDrive when SelectFirstMtdDrive calls SelectDrive, and
// SelectDrive calls SelectFirstMtdDrive, if SelectDrive is called with a drive
// that doesn't exist
function TVolumeInfo_F.AttemptSelectDrive(drive: char): boolean;
var
  drvColon: string;
  i: integer;
  foundTree: boolean;
  foundIcon: boolean;
begin
  foundTree := FALSE;
  foundIcon := FALSE;

  if drive<>#0 then
    begin
    drive := upcase(drive);
    sgrMountedSummary.row := ord(drive)-ord('A')+1;

    drvColon:= drive + ':';

    // Attempt to select on the tree view...
    tvMountedSummary.selected := tvMountedSummary.items[0];
    for i:=0 to (tvMountedSummary.items.count-1) do
      begin
      if pos(drvColon, tvMountedSummary.items[i].text)>0 then
        begin
        tvMountedSummary.selected := tvMountedSummary.items[i];
        foundTree := TRUE;
        break;
        end;
      end;

    // Attempt to select on the icon view...
    lvDrives.selected := lvDrives.items[0];
    for i:=0 to (lvDrives.items.count-1) do
      begin
      if pos(drvColon, lvDrives.items[i].Caption)>0 then
        begin
        lvDrives.selected := lvDrives.items[i];
        foundIcon := TRUE;
        break;
        end;
      end;

      
    DisplayVolumeInfo(DriveSelected());
    end;

  Result := (foundTree and foundIcon);

end;


procedure TVolumeInfo_F.SelectFirstMtdDrive();
begin
  if OTFDrives = '' then
    begin
    tvMountedSummary.selected := tvMountedSummary.items[0];
    end
  else
    begin
    AttemptSelectDrive(OTFDrives[1]);
    end;

end;

procedure TVolumeInfo_F.miRefreshClick(Sender: TObject);
var
  currDriveSelected: char;
begin
  currDriveSelected := DriveSelected();
  RefreshDisplay();
  SelectDrive(currDriveSelected);

end;

procedure TVolumeInfo_F.RefreshDisplay();
var
  i: integer;
  drvsMtd: string;
  currDrv: char;
  currDrvLC: char;
  currVolFilename: string;
  driveBits: set of 0..25;
  strVolLabel: string;
  strDispVolLabel: string;
  strNodeLabel: string;
  rootNode: TTreeNode;
  currNode: TTreeNode;
  iconIdx: integer;
  DriveType: TDriveType;
  netHostname: string;
  netSharename: string;
  anIcon: TIcon;
  shfi: SHFileInfo;
  imgListHandle: THandle;
  iconHandle: HICON;
  tmpDrivePath: string;
  iconViewListItem: TListItem;
  iconViewDriveIconNum: integer;
begin
  displayBeingRefreshed := TRUE;

  lvDrives.visible          := miViewAsIcons.checked;
  tvMountedSummary.visible  := miViewAsTree.checked;
  sgrMountedSummary.visible := miViewAsTable.checked;

  drvsMtd := OTFEUnified.DrivesMounted();

  StatusBar.panels[0].text := ReportOTFVolumes();

  sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosDrive,       0] := VOL_INF_DLG_COL_DRIVE;
  sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolLabel,    0] := VOL_INF_DLG_COL_VOLLABEL;
  sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename, 0] := VOL_INF_DLG_COL_VOLFILENAME;

  Integer(driveBits) := GetLogicalDrives();

  ClearDownTreeViewAndImages();

  // Cleardown for icon view...
  lvDrives.Items.Clear();
  ilDriveIcons.Clear();

  rootNode := tvMountedSummary.items.item[0];

  for i:=1 to 26 do
    begin
    currDrvLC := chr(i+ord('A')-1);
    currDrv:= upcase(currDrvLC);

    sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosDrive, i] := currDrv+':';

    strVolLabel := '';
    strDispVolLabel := '';
    if ((i-1) in driveBits) AND
       (miDisplayNonEncrypted.checked OR (Pos(currDrv, drvsMtd)>0)) then
      begin
      DriveType := TDriveType(GetDriveType(PChar(currDrv + ':\')));

      strVolLabel := SDUGetVolumeID(currDrv);
      strDispVolLabel := strVolLabel;

      if strVolLabel='Removable Disk' then
        begin
        // If it's a mounted volume, don't display "Removable Disk", use the
        // volume label!
        if pos(currDrv, drvsMtd)>0 then
          begin
          strDispVolLabel := SDUVolumeID(currDrv);
          end;
        end;

      if strDispVolLabel='' then
        begin
        strDispVolLabel := '<none>';
        end;


      // Now we sort out the tree view for that drive...
      strNodeLabel := strVolLabel;
      if pos('\\', strNodeLabel)=1 then
        begin
        // OK, this looks like it's a network share - format up the display
        // string to look like it does in explorer
        delete(strNodeLabel, 1, 2);
        netHostname := copy(strNodeLabel, 1, pos('\', strNodeLabel)-1);
        netSharename := copy(strNodeLabel, pos('\', strNodeLabel)+1, (length(strNodeLabel)-pos('\', strNodeLabel)));
        strNodeLabel := netSharename + ' on '''+netHostname+'''';

        if Win32Platform = VER_PLATFORM_WIN32_NT then
          begin
          strNodeLabel := lowercase(strNodeLabel);
          end;

        end
      else if (SDUInstalledOS = osWindowsNT) then
        begin
        // Under Windows NT, volume labels are lowercased, with the first
        // letter in uppercase
        if (driveType = dtFixed)   OR
           (driveType = dtCDROM)   OR
           (driveType = dtNetwork) then
          begin
          strNodeLabel := lowercase(strNodeLabel);

          if (driveType = dtFixed) OR
             (driveType = dtCDROM) then
            begin
            if length(strNodeLabel)>1 then
              begin
              strNodeLabel[1] := upcase(strNodeLabel[1]);
              end;
            end;
          end;
        end;

      // Create a suitable icon to represent the drive
      tmpDrivePath := currDrv+':\';
      imgListHandle := SHGetFileInfo(
                                     PChar(tmpDrivePath),
                                     0,
                                     shfi,
                                     sizeof(SHFileInfo),
                                     SHGFI_SYSICONINDEX or SHGFI_SMALLICON);

      iconHandle := ImageList_GetIcon(imgListHandle, shfi.iIcon, ILD_NORMAL);

      anIcon:= TIcon.Create();
      anIcon.ReleaseHandle();

      anIcon.handle := iconHandle;

      iconIdx := ilDriveTypes.addicon(anIcon);

      anIcon.ReleaseHandle();
      DestroyIcon(iconHandle);
      anIcon.Free();



      // Create the treeview entry for the drive, and setup it's icon index
      strNodeLabel := strNodeLabel +' ('+currDrv+':)';
      currNode := tvMountedSummary.Items.AddChildObject(rootNode, strNodeLabel, nil);
      currNode.ImageIndex    := iconIdx;
      currNode.SelectedIndex := iconIdx;



      // Create icon view entry for that drive...
      iconViewListItem := lvDrives.Items.Add;
      iconViewListItem.Caption := '     '+currDrv+':';
      iconViewListItem.SubItems.Add(strNodeLabel);
      iconViewDriveIconNum := AddIconForDrive(currDrv);
      if (iconViewDriveIconNum>=0) then
        begin
        iconViewListItem.ImageIndex := iconViewDriveIconNum;
        end;

      end;


    sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolLabel, i] := strDispVolLabel;

    if pos(currDrv, drvsMtd)>0 then
      begin
      currVolFilename:= OTFEUnified.GetVolFileForDrive(currDrv);
      end
    else
      begin
      currVolFilename:= '';
      end;

    sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename, i] := currVolFilename;
    end;

  tvMountedSummary.FullExpand();

  // And finally, refresh the volume info to whatever is necessary...
  DisplayVolumeInfo(DriveSelected());

  EnableDisableControls();

  displayBeingRefreshed := FALSE;

end;

procedure TVolumeInfo_F.sgrMountedSummaryClick(Sender: TObject);
begin
  DisplayVolumeInfo(DriveSelected());

end;

procedure TVolumeInfo_F.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  OTFSystemLoop: TOTFESystem;
begin

  // Note: Settings are saved when FormHide is called as a result of the form
  // closing/being hidden

  if not(STU_INI_FILE.OptUseTaskTrayIcon) then
    begin
    TheMainForm.ExitApp();
    end
  else
    begin
    // This seems to be required, otherwise an icon stays on the taskbar
    for OTFSystemLoop:=low(VolumeInfoForms) to high(VolumeInfoForms) do
      begin
      if (VolumeInfoForms[OTFSystemLoop] <> nil) then
        begin
        VolumeInfoForms[OTFSystemLoop].visible := FALSE;
        end;
      end;

    Hide();
    end;

end;

procedure TVolumeInfo_F.miExploreClick(Sender: TObject);
begin
  if miExplore.enabled then
    begin
    LaunchExplorer(DriveSelected());
    end;

end;

procedure TVolumeInfo_F.EnableDisableControls();
var
  enableControls: boolean;
  DriveNum: Integer;
  DriveBits: set of 0..25;
  currDrvLetter: char;
begin
  miClose.Visible := STU_INI_FILE.OptUseTaskTrayIcon;

  currDrvLetter := DriveSelected();

  // Make main menu menuitems invisible if not applicable
  miMountFile.visible           := OTFEUnified.Active;
  miMountPartition.visible      := OTFEUnified.Active;
  miLineVolume1.visible         := OTFEUnified.Active;
  miDismountNormal.visible      := OTFEUnified.Active;
  miDismountEmergency.visible   := OTFEUnified.Active;
  miLineVolume2.visible         := OTFEUnified.Active;
  miDismountAllNormal.visible   := OTFEUnified.Active;
  miDismountAllEmergency.visible:= OTFEUnified.Active;
  miLineVolume3.visible         := OTFEUnified.Active;
  // Make popup menu menuitems invisible if not applicable
  miMountFile2.visible        := miMountFile.visible;
  miMountPartition2.visible   := miMountPartition.visible;
  miDismountNormal2.visible   := miDismountNormal.visible;
  miDismountEmergency2.visible:= miDismountEmergency.visible;
  miLinePopupLine0.visible    := OTFEUnified.Active;
  miLinePopupLine1.visible    := OTFEUnified.Active;
  miLinePopupLine2.visible    := OTFEUnified.Active;

  // Is device mounting possible?
  miMountPartition.visible   := (miMountPartition.visible AND OTFEUnified.CanMountDevice());
  miMountPartition2.visible := miMountPartition.visible;

  enableControls := FALSE;
  if currDrvLetter<>#0 then
    begin
    enableControls := (sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename, (ord(currDrvLetter)-ord('A')+1)]<>'');
    end;

  miDismountNormal.enabled    := enableControls;
  miDismountNormal2.enabled   := miDismountNormal.enabled;
  miDismountEmergency.enabled := enableControls;
  miDismountEmergency2.enabled:= miDismountEmergency.enabled;

  miMountFile.enabled  := FALSE;
  miProperties.enabled := FALSE;
  if OTFEUnified.Active then
    begin
    // If all drive letters are in use, we can't mount anything, can we?
    Integer(DriveBits) := GetLogicalDrives;
    for DriveNum := 0 to 25 do
      begin
      if not(DriveNum in DriveBits) then
        begin
        miMountFile.enabled := TRUE;
        end
      else
        begin
        if (ord(currDrvLetter)-ord('A'))=DriveNum then
          begin
          miProperties.enabled := TRUE;
          end;
        end;

      end;

    end; // if OTFEUnified.Active then

  MountingFilesEnabled := miMountFile.enabled;

  miMountFile2.enabled      := miMountFile.enabled;
  miMountPartition.enabled  := miMountFile.enabled;
  miMountPartition2.enabled := miMountPartition.enabled;

  miProperties2.enabled := miProperties.enabled;
  miExplore.enabled     := miProperties.enabled;
  miExplore2.enabled    := miExplore.enabled;

end;

procedure TVolumeInfo_F.DismountAll(emergency: boolean);
var
  confirm: WORD;
begin
  if STU_INI_FILE.OptConfirmOnDismount then
    begin
    confirm := MessageDlg('Are you sure you wish to dismount all drives?',
                          mtConfirmation,
                          [mbYes, mbNo],
                          -1);
    end
  else
    begin
    confirm := mrYes;
    end;

  if confirm=mrYes then
    begin
    TheMainForm.OTFCryptoDismountAll(emergency);
    end;

end;


procedure TVolumeInfo_F.pnlInfoPlaceHolderResize(Sender: TObject);
begin
  RefreshVolInfoPanelSize();

end;

procedure TVolumeInfo_F.RefreshVolInfoPanelSize();
begin
  if VolumeSelected then
    begin
    VolumeInfoForms[VolumeSelectedType].Height := pnlInfoPlaceHolder.Height;
    VolumeInfoForms[VolumeSelectedType].Width  := pnlInfoPlaceHolder.Width;

    // This next bit is a bit silly, but without it, the info form doesn't
    // repaint itself properly when you shrink the window down to too small a
    // size; the Comment/Description TRichEdits are blanked out!
    VolumeInfoForms[VolumeSelectedType].Hide();
    VolumeInfoForms[VolumeSelectedType].Show();
    end;

end;

procedure TVolumeInfo_F.FormCreate(Sender: TObject);
begin
  displayBeingRefreshed := FALSE;
  VolumeSelected:= FALSE;
  MountingFilesEnabled := FALSE;

  // Let Windows know we accept dropped files
  DragAcceptFiles(self.Handle, True);
end;

procedure TVolumeInfo_F.InitDlg();
var
  OTFSystemLoop: TOTFESystem;
begin
  pnlInfoPlaceHolder.Caption := '';
  pnlInfoSpacing.Caption := '';
  pnlInfoSpacing.bevelouter := bvNone;

  VolumeInfoForms[otfesFreeOTFE  ] := TOTFVolumeDetailsFreeOTFE_F.Create(nil);
  VolumeInfoForms[otfesBestCrypt ] := TOTFVolumeDetailsBestCrypt_F.Create(nil);
  VolumeInfoForms[otfesE4M       ] := TOTFVolumeDetailsE4M_F.Create(nil);
  VolumeInfoForms[otfesTrueCrypt ] := TOTFVolumeDetailsTrueCrypt_F.Create(nil);
  VolumeInfoForms[otfesPGPDisk   ] := TOTFVolumeDetailsPGPDisk_F.Create(nil);
  VolumeInfoForms[otfesScramDisk ] := TOTFVolumeDetailsScramDisk_F.Create(nil);
  VolumeInfoForms[otfesCrossCrypt] := TOTFVolumeDetailsCrossCrypt_F.Create(nil);

  for OTFSystemLoop:=low(VolumeInfoForms) to high(VolumeInfoForms) do
    begin
    if VolumeInfoForms[OTFSystemLoop]<>nil then
      begin
      VolumeInfoForms[OTFSystemLoop].Parent := pnlInfoPlaceHolder;
      VolumeInfoForms[OTFSystemLoop].OTFComponent:= OTFEUnified.OTFComponents[VolumeInfoForms[OTFSystemLoop].OTFSystem];
      end;
    end;

  sgrMountedSummary.DefaultRowHeight := sgrMountedSummary.font.size*2;

  sgrMountedSummary.Align := alClient;
  tvMountedSummary.Align  := alClient;
  lvDrives.Align          := alClient;

  if (STU_INI_FILE.OptDlgSize_VolInfo_Height=-1) AND
     (STU_INI_FILE.OptDlgSize_VolInfo_Width=-1)  AND
     (STU_INI_FILE.OptDlgSize_VolInfo_Top=-1)    AND
     (STU_INI_FILE.OptDlgSize_VolInfo_Left=-1)   then
    begin
    self.Position := poScreenCenter;
    end
  else
    begin
    // xxx - this could be smarted up to handle the case when the user screws
    // up and manages to put the dialog off the screen, and can't get it back
    // again
    Height      := STU_INI_FILE.OptDlgSize_VolInfo_Height;
    Width       := STU_INI_FILE.OptDlgSize_VolInfo_Width;
    Top         := STU_INI_FILE.OptDlgSize_VolInfo_Top;
    Left        := STU_INI_FILE.OptDlgSize_VolInfo_Left;
    WindowState := STU_INI_FILE.OptDlgSize_VolInfo_WindowState;
    end;

  if STU_INI_FILE.OptDlgSize_VolInfo_ColWidthDrive<>-1 then
    begin
    sgrMountedSummary.ColWidths[STU_INI_FILE.OptDlgSize_VolInfo_ColPosDrive]       := STU_INI_FILE.OptDlgSize_VolInfo_ColWidthDrive;
    sgrMountedSummary.ColWidths[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolLabel]    := STU_INI_FILE.OptDlgSize_VolInfo_ColWidthVolLabel;
    sgrMountedSummary.ColWidths[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename] := STU_INI_FILE.OptDlgSize_VolInfo_ColWidthVolFilename;
    end;

  InitIconsForDrives();

  RefreshDisplay();

end;

procedure TVolumeInfo_F.sgrMountedSummaryKeyPress(Sender: TObject;
  var Key: Char);
begin
  key := (uppercase(key))[1];
  if key=#13 then
    begin
    miExploreClick(Sender);
    end
  else if (key>='A') AND (key<='Z') then
    begin
    sgrMountedSummary.row := (ord(Key)-ord('A'))+1;
    end
  else if (key=#27) then
    begin
    Close();
    end;

end;

procedure TVolumeInfo_F.sgrMountedSummaryColumnMoved(Sender: TObject;
  FromIndex, ToIndex: Integer);
begin
  STU_INI_FILE.OptDlgSize_VolInfo_ColPosDrive       := sgrMountedSummary.rows[0].IndexOf(VOL_INF_DLG_COL_DRIVE);
  STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolLabel    := sgrMountedSummary.rows[0].IndexOf(VOL_INF_DLG_COL_VOLLABEL);
  STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename := sgrMountedSummary.rows[0].IndexOf(VOL_INF_DLG_COL_VOLFILENAME);

end;

procedure TVolumeInfo_F.miPropertiesClick(Sender: TObject);
begin
  SDUShowFileProperties(DriveSelected()+':\');

end;

procedure TVolumeInfo_F.miCloseClick(Sender: TObject);
begin
  Close();

end;

procedure TVolumeInfo_F.miDismountNormalClick(Sender: TObject);
begin
  theMainForm.OTFCryptoDismountDrive(DriveSelected(), FALSE);

end;

procedure TVolumeInfo_F.miDismountEmergencyClick(Sender: TObject);
begin
  theMainForm.OTFCryptoDismountDrive(DriveSelected(), TRUE);

end;


procedure TVolumeInfo_F.DisplayVolumeInfo(drive: char);
var
  currVolType: TOTFESystem;
begin
  if displayBeingRefreshed then
    begin
    exit;
    end;

  EnableDisableControls();

  if drive=#0 then
    begin
    pnlInfoPlaceHolder.caption := '';

    VolumeSelected:= FALSE;
    BlankVolInfoDisplay();
    end
  else if sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename, (ord(drive)-ord('A')+1)]='' then
    begin
    pnlInfoPlaceHolder.caption := drive+': is not a mounted OTFE volume';

    VolumeSelected:= FALSE;
    BlankVolInfoDisplay();
    end
  else
    begin
    BlankVolInfoDisplay();
    if OTFEUnified.TypeOfEncryptedDrive(drive, currVolType) then
      begin
      if (VolumeInfoForms[currVolType] = nil) then
        begin
        pnlInfoPlaceHolder.caption := '';

        VolumeSelected:= FALSE;
        BlankVolInfoDisplay();
        end
      else
        begin
        VolumeInfoForms[currVolType].Top    := 0;
        VolumeInfoForms[currVolType].Left   := 0;
        VolumeInfoForms[currVolType].Height := pnlInfoPlaceHolder.Height;
        VolumeInfoForms[currVolType].Width  := pnlInfoPlaceHolder.Width;

        VolumeInfoForms[currVolType].Show();

        // This is required, otherwise no volume information will be displayed
        // when you start the program with 2 volumes mounted, and select the 2nd
        // volume
        VolumeInfoForms[currVolType].Top    := 0;
        VolumeInfoForms[currVolType].Left   := 0;
        VolumeInfoForms[currVolType].Height := pnlInfoPlaceHolder.Height;
        VolumeInfoForms[currVolType].Width  := pnlInfoPlaceHolder.Width;

        VolumeInfoForms[currVolType].DisplayInfoForDrive(drive);
        VolumeSelected:= TRUE;
        VolumeSelectedType:= currVolType;
        end;
      end
    else
      begin
      pnlInfoPlaceHolder.caption := 'Error: Unable to determine type of encrypted volume for drive '+drive+':';
      VolumeSelected:= FALSE;
      end;
    end;

end;

function TVolumeInfo_F.DriveSelected(): char;
var
  tmpDrv: string;
  i: integer;
begin
  Result := #0;

  if miViewAsIcons.checked then
    begin
    for i:=0 to (lvDrives.items.count-1) do
      begin
      if lvDrives.Items[i].Selected then
        begin
        // Trim the padding whitespace off the drive letter + colon
        tmpDrv := lvDrives.Items[i].Caption;
        tmpDrv := TrimLeft(tmpDrv);

        // The first letter of the item's caption is the drive letter
        if (length(tmpDrv) > 0) then
          begin
          Result := tmpDrv[1];
          break;
          end;

        end;
      end;

    end
  else if miViewAsTree.checked then
    begin
    if (tvMountedSummary.selected<>nil) AND
       (tvMountedSummary.selected<>tvMountedSummary.items[0]) then
      begin
      Result := tvMountedSummary.selected.text[length(tvMountedSummary.selected.text)-2];
      end;
    end
  else if miViewAsTable.checked then
    begin
    Result := (sgrMountedSummary.cells[STU_INI_FILE.OptDlgSize_VolInfo_ColPosDrive, sgrMountedSummary.row])[1];
    end;

end;

procedure TVolumeInfo_F.miDismountAllNormalClick(Sender: TObject);
begin
  DismountAll(FALSE);

end;

procedure TVolumeInfo_F.miDismountAllEmergencyClick(Sender: TObject);
begin
  DismountAll(TRUE);

end;

procedure TVolumeInfo_F.ListSummaryKeyPress(Sender: TObject;
  var Key: Char);
begin
  if key=#13 then
    begin
    miExploreClick(Sender);
    end
  else if key=#27 then
    begin
    key := #0;
    Close();
    end;

end;

function TVolumeInfo_F.ReportOTFVolumes(): string;
var
  i: integer;
  drvsList: string;
begin
  lvDrives.visible          := miViewAsIcons.checked;
  tvMountedSummary.visible  := miViewAsTree.checked;
  sgrMountedSummary.visible := miViewAsTable.checked;

  OTFDrives := OTFEUnified.DrivesMounted();

  if OTFDrives='' then
    begin
    Result := 'There are currently no OTFE volumes mounted';
    miDismountAllNormal.enabled    := FALSE;
    miDismountAllEmergency.enabled := FALSE;
    end
  else
    begin
    miDismountAllNormal.enabled    := TRUE;
    miDismountAllEmergency.enabled := TRUE;
    drvsList := '';
    for i:=1 to length(OTFDrives) do
      begin
      if i>1 then
        begin
        drvsList := drvsList+', ';
        end;
      drvsList := drvsList + OTFDrives[i] + ':';
      end;

    if length(OTFDrives)=1 then
      begin
      Result := 'Drive '+drvsList+' is a mounted OTFE volume';
      end
    else
      begin
      Result := 'Drives '+drvsList+' are mounted OTFE volumes';
      end;

    end;
end;


procedure TVolumeInfo_F.tvMountedSummaryChange(Sender: TObject;
  Node: TTreeNode);
begin
  DisplayVolumeInfo(DriveSelected());

end;

procedure TVolumeInfo_F.WMDeviceChange(var Msg : TMessage);
var
  currDrive: char;
  priorOTFDrives: string;
  i: integer;
  junk: boolean;
begin
  currDrive:= DriveSelected();
  priorOTFDrives := OTFDrives;

  RefreshDisplay();

  SelectDrive(currDrive);

  if priorOTFDrives<>OTFDrives then
    begin
    for i:=1 to length(OTFDrives) do
      begin
      if pos(OTFDrives[i], priorOTFDrives)<1 then
        begin
        SelectDrive(OTFDrives[i]);
        break;
        end;
      end;
    end;

  theMainForm.UpdateTrayIcon();

  // This fixes a bug in which a SecureTrayUtil "window" remains on the
  // tasktray after the user has started SecureTrayUtil and successfully
  // mounted a PGPDisk volume by the "display mount dialog after starting"
  // option
  if not(visible) then
    begin
    FormCloseQuery(self, junk);
    end;

end;

procedure TVolumeInfo_F.miMountFileClick(Sender: TObject);
begin
  TheMainForm.OTFCryptoMountPrompted();

end;

procedure TVolumeInfo_F.miMountPartitionClick(Sender: TObject);
begin
  TheMainForm.OTFCryptoMountPartition();

end;

procedure TVolumeInfo_F.SaveSettings();
begin
  STU_INI_FILE.OptDlgSize_VolInfo_Height      := Height;
  STU_INI_FILE.OptDlgSize_VolInfo_Width       := Width;
  STU_INI_FILE.OptDlgSize_VolInfo_Top         := Top;
  STU_INI_FILE.OptDlgSize_VolInfo_Left        := Left;

  STU_INI_FILE.OptDlgSize_VolInfo_WindowState := WindowState;

  // The changing of the column order is handled by one of the
  // sgrMountedSummary events

  STU_INI_FILE.OptDlgSize_VolInfo_IconView  := miViewAsIcons.checked;
  STU_INI_FILE.OptDlgSize_VolInfo_TreeView  := miViewAsTree.checked;
  STU_INI_FILE.OptDlgSize_VolInfo_TableView := miViewAsTable.checked;
  STU_INI_FILE.OptDlgSize_VolInfo_DispNonEncrypted := miDisplayNonEncrypted.checked;
  STU_INI_FILE.OptDlgSize_VolInfo_DispVolInfoPnl := miDisplayVolInfo.checked;

  STU_INI_FILE.OptDlgSize_VolInfo_ColWidthDrive       := sgrMountedSummary.ColWidths[STU_INI_FILE.OptDlgSize_VolInfo_ColPosDrive];
  STU_INI_FILE.OptDlgSize_VolInfo_ColWidthVolLabel    := sgrMountedSummary.ColWidths[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolLabel];
  STU_INI_FILE.OptDlgSize_VolInfo_ColWidthVolFilename := sgrMountedSummary.ColWidths[STU_INI_FILE.OptDlgSize_VolInfo_ColPosVolFilename];

  STU_INI_FILE.WriteIniFile();

end;


procedure TVolumeInfo_F.FormDestroy(Sender: TObject);
var
  OTFSystemLoop: TOTFESystem;
begin
  // Form settings would have been saved when the form was closed

  for OTFSystemLoop:=low(VolumeInfoForms) to high(VolumeInfoForms) do
    begin
    if VolumeInfoForms[OTFSystemLoop]<>nil then
      begin
      VolumeInfoForms[OTFSystemLoop].Free();
      VolumeInfoForms[OTFSystemLoop] := nil;
      end;
    end;

end;

procedure TVolumeInfo_F.FormHide(Sender: TObject);
begin
  SaveSettings();

end;

procedure TVolumeInfo_F.FormResize(Sender: TObject);
begin
  StatusBar.align := alBottom;

end;

procedure TVolumeInfo_F.miDisplayVolInfoClick(Sender: TObject);
begin
  TMenuitem(Sender).checked := not(TMenuitem(Sender).checked);
  SetDisplayVolInfoPanel(TMenuitem(Sender).checked);

end;

procedure TVolumeInfo_F.SetDisplayVolInfoPanel(display: boolean);
begin
  if display<>pnlInfoSpacing.visible then
    begin
    if pnlInfoSpacing.visible then
      begin
      pnlInfoSpacing.visible := display;
      width := width - pnlInfoSpacing.width;
      end
    else
      begin
      width := width + pnlInfoSpacing.width;
      pnlInfoSpacing.visible := display;
      end;
    end;

  miDisplayVolInfo.checked := display;

  StatusBar.align := alBottom;
  RefreshVolInfoPanelSize();

end;

procedure TVolumeInfo_F.miExitClick(Sender: TObject);
begin
  TheMainForm.ExitApp();

end;

procedure TVolumeInfo_F.FormActivate(Sender: TObject);
begin
  // This is required because otherwise, when the Main_F does a Show() on
  // another form, the menu is unmerged!
  muMainMenu.Merge(TheMainForm.muCommonMenu);

end;

// This is required to ensure that when the window is minimized, it shrinks
// down to the taskbar, and not just above it
procedure TVolumeInfo_F.WMSysCommand(var msg: TWMSysCommand);
begin
  if (msg.CmdType and $FFF0 = SC_MINIMIZE) then
    begin
    Application.Minimize();
    end
  else
    begin
    inherited;
    end;

end;

// Handle files being dropped onto the form from (for example) Windows Explorer.
// Files dropped onto the form should be treated as though the user is trying
// to mount them as volume files
procedure TVolumeInfo_F.WMDropFiles(var Msg: TWMDropFiles);
var
  buffer: array[0..MAX_PATH] of Char;
  numFiles: longint;
  i: integer;
  filesToMount: TStringList;
begin
  if MountingFilesEnabled then
    begin
    try
      // Bring our window to the foreground
      SetForeGroundWindow(self.handle);

      // Handle the dropped files
      numFiles := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
      filesToMount := TStringList.Create();
      try
        for i:=0 to (numFiles-1) do
          begin
          DragQueryFile(msg.Drop,
                        i,
                        @buffer,
                        sizeof(buffer));
          filesToMount.Add(buffer);
          end;

        TheMainForm.OTFCryptoMountUnPrompted(filesToMount, FALSE);

        Msg.Result := 0;
      finally
        filesToMount.Free();
      end;
    finally
      DragFinish(Msg.Drop);
    end;
    end;

end;


procedure TVolumeInfo_F.FormShow(Sender: TObject);
begin
  miViewAsIcons.checked := STU_INI_FILE.OptDlgSize_VolInfo_IconView;
  miViewAsTree.checked  := STU_INI_FILE.OptDlgSize_VolInfo_TreeView;
  miViewAsTable.checked := STU_INI_FILE.OptDlgSize_VolInfo_TableView;

  miDisplayNonEncrypted.checked := STU_INI_FILE.OptDlgSize_VolInfo_DispNonEncrypted;

  SetDisplayVolInfoPanel(STU_INI_FILE.OptDlgSize_VolInfo_DispVolInfoPnl);

  RefreshDisplay();

  // Position the cursor on the first mounted-volume drive
  SelectFirstMtdDrive();

  muMainMenu.Merge(TheMainForm.muCommonMenu);

end;

procedure TVolumeInfo_F.BlankVolInfoDisplay();
var
  OTFSystemLoop: TOTFESystem;
begin
  for OTFSystemLoop:=low(VolumeInfoForms) to high(VolumeInfoForms) do
    begin
    if VolumeInfoForms[OTFSystemLoop]<>nil then
      begin
      VolumeInfoForms[OTFSystemLoop].Hide();
      end;
    end;

end;


// Clear down the tree view, and all images stored in the imagelist it uses
procedure TVolumeInfo_F.ClearDownTreeViewAndImages();
var
  anIcon: TIcon;
  shfi: SHFileInfo;
  rootNode: TTreeNode;
  imgListHandle: THandle;
  iconHandle: HICON;
  pidl: PITEMIDLIST;
  iconIdx: integer;
begin
  tvMountedSummary.items.clear();
  ilDriveTypes.clear();

  SHGetSpecialFolderLocation(0, CSIDL_DRIVES, pidl);

  imgListHandle := SHGetFileInfo(
                                 PChar(pidl),
                                 0,
                                 shfi,
                                 sizeof(SHFileInfo),
                                 SHGFI_PIDL or SHGFI_SYSICONINDEX or SHGFI_SMALLICON);

  iconHandle := ImageList_GetIcon(imgListHandle, shfi.iIcon, ILD_NORMAL);

  anIcon:= TIcon.Create();
  anIcon.ReleaseHandle();

  anIcon.handle := iconHandle;

  iconIdx := ilDriveTypes.addicon(anIcon);

  anIcon.ReleaseHandle();
  DestroyIcon(iconHandle);
  anIcon.Free();


  rootNode := tvMountedSummary.Items.AddChildObject(nil, 'My Computer', nil);

  rootNode.ImageIndex    := iconIdx;
  rootNode.SelectedIndex := iconIdx;

end;



// This procedure is ripped from the FreeOTFE GUI
procedure TVolumeInfo_F.InitIconsForDrives();
var
  tmpColumn: TListColumn;
begin
  // Listview setup...
  lvDrives.SmallImages := ilDriveIcons;
  lvDrives.RowSelect:= TRUE;
  lvDrives.MultiSelect:= FALSE; // NOTE: THIS DIFFERS FROM THE FREEOTFE GUI
  lvDrives.showcolumnheaders := TRUE;
  lvDrives.ReadOnly := TRUE;
  lvDrives.ViewStyle := vsReport;
//    lvDrives.ViewStyle := vsIcon;
//    lvDrives.ViewStyle := vsList;
  lvDrives.IconOptions.Arrangement := iaTop;


  // Listview columns...
  tmpColumn := lvDrives.Columns.Add;
  tmpColumn.Caption := 'Drive';
  tmpColumn.width := 100;

//    tmpColumn.minwidth := lvDrives.width;
  tmpColumn := lvDrives.Columns.Add;
  tmpColumn.Caption := 'Volume';
  tmpColumn.width := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;


// xxx - shouldn't this work?     NewColumn.autosize := TRUE;
// xxx - shouldn't this work?     NewColumn.minwidth := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;


// xxx - use destroyimage or whatever it was to cleardown the **imagelist**

end;


// This procedure is ripped from the FreeOTFE GUI
// Add an icon to represent the specified drive
// Returns: The icon's index in ilDriveIcons, or -1 on error
function TVolumeInfo_F.AddIconForDrive(driveLetter: char): integer;
var
  iconIdx: integer;
  anIcon: TIcon;
  shfi: SHFileInfo;
  imgListHandle: THandle;
  iconHandle: HICON;
  tmpDrivePath: string;
begin
  iconIdx := -1;
  
  // Create a suitable icon to represent the drive
  tmpDrivePath := driveLetter+':\';
  imgListHandle := SHGetFileInfo(
                                 PChar(tmpDrivePath),
                                 0,
                                 shfi,
                                 sizeof(shfi),
                                 SHGFI_SYSICONINDEX or SHGFI_LARGEICON);
//                                 SHGFI_DISPLAYNAME

  if imgListHandle<>0 then
    begin
    iconHandle := ImageList_GetIcon(imgListHandle, shfi.iIcon, ILD_NORMAL);
    if (iconHandle<>0) then
      begin
      anIcon:= TIcon.Create();
      try
        anIcon.ReleaseHandle();

        anIcon.handle := iconHandle;

        if (ilDriveIcons.Width<anIcon.Width) then
          begin
          ilDriveIcons.Width := anIcon.Width;
          end;
        if (ilDriveIcons.Height<anIcon.Height) then
          begin
          ilDriveIcons.Height := anIcon.Height;
          end;

        iconIdx := ilDriveIcons.addicon(anIcon);

        anIcon.ReleaseHandle();
        DestroyIcon(iconHandle);
      finally
        anIcon.Free();
      end;

      end;
    end;


  Result := iconIdx;

end;


procedure TVolumeInfo_F.ViewChangeClick(Sender: TObject);
begin
  TMenuitem(Sender).checked := not(TMenuitem(Sender).checked);

  RefreshDisplay();

end;

procedure TVolumeInfo_F.miDisplayNonEncryptedClick(Sender: TObject);
begin
  TMenuitem(Sender).checked := not(TMenuitem(Sender).checked);
  miRefreshClick(Sender);

end;

procedure TVolumeInfo_F.lvDrivesChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  DisplayVolumeInfo(DriveSelected());

end;

procedure TVolumeInfo_F.lvDrivesClick(Sender: TObject);
begin
  DisplayVolumeInfo(DriveSelected());

end;


// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
{$WARN UNIT_PLATFORM ON}
// -----------------------------------------------------------------------------

END.


