unit STUShellExt_U;
// Description: SecureTrayUtil Shell Extensions
// Description:
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.SDean12.org/
//
// -----------------------------------------------------------------------------
//
// Shell Extensions for SecureTrayUtil


interface

uses
  Windows, ActiveX, ComObj, ShlObj;

type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
  private
    FFileName: array of array [0..MAX_PATH] of Char;
    function GetMainAppExe(): string;
    procedure RunCommand(runCmdLine: string);
  protected
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
    function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
  end;

const
  Class_ContextMenu: TGUID = '{6A64A050-1803-11D3-AB38-00104C39B8EC}';

  MENUOFFSET_MOUNT     = 1; // Doubles up as dismount
  MENUOFFSET_DISMOUNT  = 2; // Doubles up as mount
  MENUOFFSET_DESTROY   = 3;
  MENUOFFSET_SHREDFREE = 4;
  MENUOFFSET_TOUCH     = 5;
  MENUOFFSET_HASH      = 6;

  MAX_NUMBER_MENUITEMS_ADDED = 7; // This MUST always be set to the highest
                                  // number + 1

implementation

uses ComServ, SysUtils, ShellApi, Registry, RegStr,
     Dialogs,
     AppGlobalConsts_U,
     AppGlobal_U,
     STUIniFile_U,
     SDUGeneral,
     OTFEUnified_U;

// Perform initialization
function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
  i: integer;
begin
  // Fail the call if lpdobj is Nil.
  if (lpdobj = nil) then begin
    Result := E_INVALIDARG;
    Exit;
  end;

  with FormatEtc do begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;

  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lpdobj.GetData(FormatEtc, StgMedium);
  if Failed(Result) then
    Exit;
  // If only one file is selected, retrieve the file name and store it in
  // FFileName. Otherwise fail the call.
  SetLength(FFilename, 0);
  SetLength(FFilename, DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0));
  for i:=low(FFilename) to high(FFilename) do
    begin
    DragQueryFile(StgMedium.hGlobal, i, FFileName[i], SizeOf(FFileName[i]));
    end;
    Result := NOERROR;

  ReleaseStgMedium(StgMedium);
end;


// Function called by the shell when it wants us to add our menuitems
function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
          idCmdLast, uFlags: UINT): HResult;
var
  menuIndexOffset: UINT;
  fileSelected: boolean;
  dirSelected: boolean;
  destroyMenuitemString: string;
  plural: string;
  i: integer;

  addMenuItemsMenu: HMENU;
  addMenuItemsMenuIndex: UINT;
  abortThisItem: boolean;
  firstStr: string;
  UnifiedDiskEncryption: TOTFEUnified;
  strTouchMenuName: string;

  fileIsVolumeFile: boolean;
  fileIsMountedVolume: boolean;

  tempStr: string;
begin
  Result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);
  menuIndexOffset := 0;

  firstStr := FFilename[low(FFilename)];

  if ((uFlags and $0000000F) = CMF_NORMAL) or
     ((uFlags and CMF_EXPLORE) <> 0) then
    begin
    STU_INI_FILE := TSTUIniFile.Create(INI_FILENAME);
    try
      // Read in the ini file/setup it's defaults.
      STU_INI_FILE.ReadIniFile();

      if STU_INI_FILE.OptShellExtMenuSeparators then
        begin
        InsertMenu(Menu,
                   indexMenu + menuIndexOffset,
                   MF_SEPARATOR or MF_BYPOSITION,
                   0,
                   nil);
        inc(menuIndexOffset);
        end;

      if not(STU_INI_FILE.OptShellExtSubMenu) then
        begin
        addMenuItemsMenu := Menu;
        addMenuItemsMenuIndex := indexMenu + menuIndexOffset;
        end
      else
        begin
        addMenuItemsMenu := CreatePopupMenu();
        InsertMenu(Menu,
                   indexMenu+menuIndexOffset,
                   MF_POPUP or MF_BYPOSITION,
                   addMenuItemsMenu,
                   PChar(STU_INI_FILE.OptShellExtSubMenuTitle));
        addMenuItemsMenuIndex := 0;
        inc(menuIndexOffset);
        end;

      plural := '';
      if (low(FFilename)<>high(FFilename)) then
        begin
        plural := 's';
        end;

      if STU_INI_FILE.OptShellExtMenuMountVolume then
        begin
        // Only add the "Mount as volume file..." menuitem if only 1 item is
        // selected, and it's a file
        if plural='' then
          begin
          UnifiedDiskEncryption:= TOTFEUnified.Create(nil);
          try
            try
              UnifiedDiskEncryption.Active := TRUE;
            except
              // Nothing
            end;
            if UnifiedDiskEncryption.Active then
              begin
              fileIsVolumeFile := UnifiedDiskEncryption.IsEncryptedVolFile(firstStr);
              fileIsMountedVolume := (UnifiedDiskEncryption.GetDriveForVolFile(firstStr)<>#0);

              if (fileIsVolumeFile) AND not(fileIsMountedVolume)then
                begin
                InsertMenu(addMenuItemsMenu,
                           addMenuItemsMenuIndex,
                           MF_STRING or MF_BYPOSITION,
                           idCmdFirst+MENUOFFSET_MOUNT,
                           'Mount as &volume file...');
                inc(addMenuItemsMenuIndex);
                end
              // If it's a mounted drive, or mounted volume file
              else if ((UnifiedDiskEncryption.GetVolFileForDrive( (FFilename[low(FFilename)])[0] )<>'') AND
                       (firstStr[1]=':')  AND
                       (firstStr[2]='\')  AND
                       (firstStr[3]=#0) ) OR
                      (fileIsMountedVolume) then
                begin
                // We're on the root of a mounted volume drive, allow option of dismounting...
                InsertMenu(addMenuItemsMenu,
                           addMenuItemsMenuIndex,
                           MF_STRING or MF_BYPOSITION,
                           idCmdFirst+MENUOFFSET_DISMOUNT,
                           'Dismount &volume...');
                inc(addMenuItemsMenuIndex);
                end;
              end;
          finally
            UnifiedDiskEncryption.Free();
          end; // try
          end; // if plural='' then
        end; // if STU_INI_FILE.OptShellExtMenuMountVolume then

      if STU_INI_FILE.OptShellExtMenuShredFreeSpace then
        begin
        // Only add the "Shred free space" menuitem if only 1 item is
        // selected, and it's a drive
        if (plural='')                           AND
           (IsADir(firstStr))   AND
           (length(firstStr)=3) then
          begin
          InsertMenu(addMenuItemsMenu,
                     addMenuItemsMenuIndex,
                     MF_STRING or MF_BYPOSITION,
                     idCmdFirst+MENUOFFSET_SHREDFREE,
                     '&Shred free space');
          inc(addMenuItemsMenuIndex);
          end;
        end;

      fileSelected := FALSE;
      dirSelected := FALSE;
      if (STU_INI_FILE.OptShellExtMenuShredFile) OR
         (STU_INI_FILE.OptShellExtMenuShredDir)  then
        begin
        abortThisItem := FALSE;
        for i:=low(FFileName) to high(FFileName) do
          begin
          if IsADir(FFilename[i]) then
            begin
            // If the user's selected the root directory anywhere, don't show
            // this menuitem - it's too dangerous
            tempStr := FFilename[i];
            if length(tempStr)=3 then
              begin
              abortThisItem := TRUE;
              break;
              end;

            dirSelected := TRUE;
            end
          else
            begin
            fileSelected := TRUE;
            end;
          end;

        fileSelected := fileSelected AND STU_INI_FILE.OptShellExtMenuShredFile;
        dirSelected  := dirSelected AND STU_INI_FILE.OptShellExtMenuShredDir;

        if fileSelected AND dirSelected then
          begin
          destroyMenuitemString := '&Shred selected...';
          end
        else if fileSelected then
          begin
          destroyMenuitemString := '&Shred  file'+plural+'...';
          end
        else if dirSelected then
          begin
          destroyMenuitemString := '&Shred directory'+plural+'...';
          end
        else
          begin
          abortThisItem := TRUE;
          end;

        if not(abortThisItem) then
          begin
          InsertMenu(addMenuItemsMenu,
                     addMenuItemsMenuIndex,
                     MF_STRING or MF_BYPOSITION,
                     idCmdFirst+MENUOFFSET_DESTROY,
                     PChar(destroyMenuitemString));
          inc(addMenuItemsMenuIndex);
          end;
        end;


      fileSelected := FALSE;
      dirSelected := FALSE;
      if (STU_INI_FILE.OptShellExtMenuTouch) then
        begin
        abortThisItem := FALSE;
        for i:=low(FFileName) to high(FFileName) do
          begin
          if IsADir(FFilename[i]) then
            begin
            // If the user's selected the root directory anywhere, don't show
            // this menuitem - it's silly
            tempStr := FFilename[i];
            if length(tempStr)=3 then
              begin
              abortThisItem := TRUE;
              break;
              end;

            dirSelected := TRUE;
            end
          else
            begin
            fileSelected := TRUE;
            end;
          end;

        if fileSelected AND dirSelected then
          begin
          strTouchMenuName := 'To&uch selected...';
          end
        else if fileSelected then
          begin
          strTouchMenuName := 'To&uch file'+plural+'...';
          end
        else if dirSelected then
          begin
          strTouchMenuName := 'To&uch directory'+plural+'...';
          end
        else
          begin
          abortThisItem := TRUE;
          end;

        if not(abortThisItem) then
          begin
          InsertMenu(addMenuItemsMenu,
                     addMenuItemsMenuIndex,
                     MF_STRING or MF_BYPOSITION,
                     idCmdFirst+MENUOFFSET_TOUCH,
                     PChar(strTouchMenuName));

          inc(addMenuItemsMenuIndex);
          end;
        end;


      if STU_INI_FILE.OptShellExtMenuHash then
        begin
        if (plural='')           AND
           not(IsADir(firstStr)) then
          begin
          InsertMenu(addMenuItemsMenu,
                     addMenuItemsMenuIndex,
                     MF_STRING or MF_BYPOSITION,
                     idCmdFirst+MENUOFFSET_HASH,
                     '&Calculate file hash');
          inc(addMenuItemsMenuIndex);
          end;
        end;


      if STU_INI_FILE.OptShellExtMenuSeparators then
        begin
        if STU_INI_FILE.OptShellExtSubMenu then
          begin
          addMenuItemsMenuIndex := menuIndexOffset;
          end;

        InsertMenu(Menu,
                   addMenuItemsMenuIndex,
                   MF_SEPARATOR or MF_BYPOSITION,
                   0,
                   nil);
        end;

      // Return number of menu items added
      Result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, MAX_NUMBER_MENUITEMS_ADDED)
    finally
      STU_INI_FILE.Free();
    end; // try...finally
    end;

end;


// Function called by the shell to invoke one of out menuitems
function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
var
  i: integer;
  runCmdLine: string;
  UnifiedDiskEncryption: TOTFEUnified;
begin
  // Make sure we are not being called by an application
  if (HiWord(Integer(lpici.lpVerb)) <> 0) then
    begin
    Result := E_FAIL;
    Exit;
    end;

  if (LoWord(lpici.lpVerb) = MENUOFFSET_MOUNT) then
    begin
    runCmdLine := GetMainAppExe() + ' ';
    runCmdLine := runCmdLine + '/mount ' + SDUConvertLFNToSFN(FFilename[low(FFileName)]);
    RunCommand(runCmdLine);
    end
  else if (LoWord(lpici.lpVerb) = MENUOFFSET_DISMOUNT) then
    begin
    runCmdLine := GetMainAppExe() + ' ';
    if IsADir(FFilename[low(FFileName)]) then
      begin
      runCmdLine := runCmdLine + '/dismount ' + FFilename[low(FFileName)];
      end
    else
      begin
      UnifiedDiskEncryption:= TOTFEUnified.Create(nil);
      try
        try
          UnifiedDiskEncryption.Active := TRUE;
        except
          // Nothing
        end;
        if UnifiedDiskEncryption.Active then
          begin
          runCmdLine := runCmdLine + '/dismount ';
          runCmdLine := runCmdLine + UnifiedDiskEncryption.GetDriveForVolFile(FFilename[low(FFilename)]);
          end;
      finally
        UnifiedDiskEncryption.Free();
      end;

      end;
    RunCommand(runCmdLine);
    end
  else if (LoWord(lpici.lpVerb) = MENUOFFSET_DESTROY) then
    begin
    runCmdLine := GetMainAppExe() + ' /shred ';
    for i:=low(FFileName) to high (FFileName) do
      begin
      runCmdLine := runCmdLine + ' ' + SDUConvertLFNToSFN(FFilename[i]);
      end;
    RunCommand(runCmdLine);
    end
  else if (LoWord(lpici.lpVerb) = MENUOFFSET_TOUCH) then
    begin
    runCmdLine := GetMainAppExe() + ' /promptdatetime /touch ';
    for i:=low(FFileName) to high (FFileName) do
      begin
      runCmdLine := runCmdLine + ' ' + SDUConvertLFNToSFN(FFilename[i]);
      end;
    RunCommand(runCmdLine);
    end
  else if (LoWord(lpici.lpVerb) = MENUOFFSET_HASH) then
    begin
    runCmdLine := GetMainAppExe() + ' /hash ' + SDUConvertLFNToSFN(FFilename[low(FFileName)]);
    RunCommand(runCmdLine);
    end
  else if (LoWord(lpici.lpVerb) = MENUOFFSET_SHREDFREE) then
    begin
    runCmdLine := GetMainAppExe() + ' /shredfree ' + (FFilename[low(FFileName)])[0];
    RunCommand(runCmdLine);
    end
  else
    begin
    Result := E_INVALIDARG;
    Exit;
    end;

  Result := NOERROR;

end;


// Function called by the shell to obtain status (help) text, etc
function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
begin
  if (idCmd = MENUOFFSET_MOUNT) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Mount the selected OTFE volume file/drive');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_DISMOUNT) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Dismount encrypted volume');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_DESTROY) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Destroy selected files/directories');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_TOUCH) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Touch selected files/directories');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_HASH) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Calculate hash of selected file');
      end;
    Result := NOERROR;
    end
  else
    begin
    Result := E_INVALIDARG;
    end;
end;


function TContextMenu.GetMainAppExe(): string;
var
  registry: TRegistry;
  keyName: string;
begin
  registry := TRegistry.create();
  try
    registry.RootKey := HKEY_LOCAL_MACHINE;
    keyName := REGSTR_PATH_APPPATHS + '\SecureTrayUtil.exe';
    if not(registry.OpenKey(keyName, FALSE)) then
      begin
      showmessage('Unable to get Application Path to SecureTrayUtil.exe');
      end
    else
      begin
      Result := registry.ReadString('');
      registry.CloseKey;
      end;
  finally
    registry.Free();
  end;
end;


procedure TContextMenu.RunCommand(runCmdLine: string);
begin
  if (WinExec(PChar(runCmdLine), SW_RESTORE))<31 then
    begin
    ErrorMessage('STU: Error running command');
    end;

end;


type
  TContextMenuFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

// Register/Unregister this DLL
// Note: AppPath registration is assumed - done in the main app
procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
const
  REGKEY_ALLEXT = '*\shellex\ContextMenuHandlers\SecureTrayUtil';
  REGKEY_DIR    = 'Directory\shellex\ContextMenuHandlers\SecureTrayUtil';
  REGKEY_DRIVE  = 'Drive\shellex\ContextMenuHandlers\SecureTrayUtil';
var
  ClassID: string;
  registry: TRegistry;
begin
  inherited UpdateRegistry(Register);
  
  registry := TRegistry.create();
  STU_INI_FILE := TSTUIniFile.Create(INI_FILENAME);
  try
    // Read in the ini file/setup it's defaults.
    STU_INI_FILE.ReadIniFile();

    registry.RootKey := HKEY_CLASSES_ROOT;

    try
      if Register then
        begin
        ClassID := GUIDToString(Class_ContextMenu);

        if (STU_INI_FILE.OptShellExtMenuMountVolume) OR
           (STU_INI_FILE.OptShellExtMenuTouch)       OR
           (STU_INI_FILE.OptShellExtMenuHash)        OR
           (STU_INI_FILE.OptShellExtMenuShredFile)   then
           begin
           registry.OpenKey(REGKEY_ALLEXT, TRUE);
           registry.WriteString('', ClassID);
           registry.CloseKey();
           end;

        if STU_INI_FILE.OptShellExtMenuShredDir then
          begin
          registry.OpenKey(REGKEY_DIR, TRUE);
          registry.WriteString('', ClassID);
          registry.CloseKey();
          end;

        if STU_INI_FILE.OptShellExtMenuShredFreeSpace then
          begin
          registry.OpenKey(REGKEY_DRIVE, TRUE);
          registry.WriteString('', ClassID);
          registry.CloseKey();
          end;

        if (Win32Platform = VER_PLATFORM_WIN32_NT) then
          with TRegistry.Create do
            try
              RootKey := HKEY_LOCAL_MACHINE;
              OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extensions', True);
              OpenKey('Approved', True);
              WriteString(ClassID, 'SecureTrayUtil Shell Extensions');
            finally
              Free;
            end;
        end
      else
        begin
        registry.DeleteKey(REGKEY_ALLEXT);
        registry.DeleteKey(REGKEY_DIR);
        registry.DeleteKey(REGKEY_DRIVE);
        end;

    except
      on ERegistryException do
        begin
        showmessage('An error occured while writing to the registry; if you are running under NT, please ensure that you have the necessary permissions do perform this operation (i.e. logon as administrator) and try again');
        end;
    end;

  finally
    STU_INI_FILE.Free();
    registry.Free();
  end;

end;


initialization
  TContextMenuFactory.Create(ComServer,
                             TContextMenu,
                             Class_ContextMenu,
                             '',
                             'SecureTrayUtil Shell Extensions',
                             ciMultiInstance,
                             tmApartment);
END.


