unit Main_U;
// This is a simple little application to test reading data in from a port
// Reads data in one char at a time, and displays it in it's decimal format
// Written: 23rd June 1999

// Credit due to Peter Johnson (pjohnson@netspace.net.au) for his article on
// how to use serial ports under Delphi that can be found at:
// http://www.innotts.co.uk/~zephyr/serial.html
// Thanks Pete!


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls,
  COMPort_U, XPMan;

const
  APP_VERSION = 'v1.01';

type
  TMain_F = class(TForm)
    ListBox1: TListBox;
    ttAutoPoll: TTimer;
    pbClear: TButton;
    pbClose: TButton;
    Bevel1: TBevel;
    pcSteps: TPageControl;
    tsPortSelect: TTabSheet;
    tsTestPort1: TTabSheet;
    tsClosePort: TTabSheet;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    cbPorts: TComboBox;
    GroupBox4: TGroupBox;
    Label6: TLabel;
    pbPoll: TButton;
    GroupBox3: TGroupBox;
    Label3: TLabel;
    pbClosePort: TButton;
    ckAutoPoll: TCheckBox;
    Label8: TLabel;
    ttAutoTXRXRand: TTimer;
    GroupBox1: TGroupBox;
    Label2: TLabel;
    pbSetupPort: TButton;
    Label10: TLabel;
    tsInstructions: TTabSheet;
    RichEdit1: TRichEdit;
    Label11: TLabel;
    lblVersion: TLabel;
    tsTestPort2: TTabSheet;
    GroupBox5: TGroupBox;
    Label12: TLabel;
    Label13: TLabel;
    Label19: TLabel;
    pbTXRX: TButton;
    pbTXRXRandomPair: TButton;
    ckAutoTXRXRand: TCheckBox;
    Label17: TLabel;
    lblInterval: TLabel;
    edAutoInterval: TEdit;
    lblIntervalUnits: TLabel;
    pbSetInterval: TButton;
    Label4: TLabel;
    pbSendNow: TButton;
    edUserText: TEdit;
    rgTXRXPairBeep: TRadioGroup;
    Label5: TLabel;
    Label7: TLabel;
    XPManifest1: TXPManifest;
    procedure pbSetupPortClick(Sender: TObject);
    procedure pbPollClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbClosePortClick(Sender: TObject);
    procedure ckAutoPollClick(Sender: TObject);
    procedure pbClearClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure pbCloseClick(Sender: TObject);
    procedure pbSetIntervalClick(Sender: TObject);
    procedure pbTXRXClick(Sender: TObject);
    procedure pbTXRXRandomPairClick(Sender: TObject);
    procedure ckAutoTXRXRandClick(Sender: TObject);
    procedure pbSendNowClick(Sender: TObject);
  private
    testPort: TCOMPort;
    procedure FillCOMPortsComboBox();
    procedure EnableDisableControls(enable: boolean);
  public
    { Public declarations }
  end;

var
  Main_F: TMain_F;
  ComFile: THandle;

implementation

{$R *.DFM}

uses AppGlobal_U,
     SDUGeneral;

procedure TMain_F.FillCOMPortsComboBox();
begin
  testPort.GetPorts(TStringList(cbPorts.items));

end;

procedure TMain_F.pbSetupPortClick(Sender: TObject);
var
  setupOK: boolean;
  portNumberString: string;
begin
  portNumberString := cbPorts.text[length(cbPorts.text)];
  setupOK := testPort.SetupCOMPort(strtoint(portNumberString));

  if setupOK then
    begin
    ListBox1.Items.add('Opened port OK.');
    end
  else
    begin
    ListBox1.Items.add('Couldn''t open port');
    end;

  EnableDisableControls(setupOK);

end;

procedure TMain_F.pbPollClick(Sender: TObject);
var
  data: string;
  addString: string;
begin
  addString := 'Failed to read data from port';

  if testPort.ReadData(data, 1)<>-1 then
    begin
    if length(data)>0 then
      begin
      addString := 'Read data char: #'+inttostr(ord(data[1]));
      if (data[1] in ['a'..'z']) OR
         (data[1] in ['A'..'Z']) OR
         (data[1] in ['0'..'9']) then
        begin
        addString := addString + ' ('+data+')';
        end;
      end
    else
      begin
      addString := 'No chars to read';
      end;
    end;

  ListBox1.Items.add(addString);
end;

procedure TMain_F.FormCreate(Sender: TObject);
begin
  lblVersion.caption := APP_VERSION;

  pcSteps.activepage := tsInstructions;

  testPort:= TCOMPort.Create();

  FillCOMPortsComboBox();
  if cbPorts.items.count<1 then
    begin
    ListBox1.Items.add('No serial ports detected');
    end
  else
    begin
    cbPorts.itemindex := 0;
    end;

end;

procedure TMain_F.pbClosePortClick(Sender: TObject);
begin
  testPort.CloseCOMPort();
  ListBox1.Items.add('Closed port.');
  EnableDisableControls(FALSE);

end;

procedure TMain_F.ckAutoPollClick(Sender: TObject);
begin
  ttAutoPoll.enabled := ckAutoPoll.checked;

  SDUEnableControl(edAutoInterval, (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked)));
  SDUEnableControl(pbSetInterval, (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked)));
  SDUEnableControl(lblInterval, (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked)));
  SDUEnableControl(lblIntervalUnits, (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked)));

end;

procedure TMain_F.pbClearClick(Sender: TObject);
begin
  ListBox1.items.clear;

end;

procedure TMain_F.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  testPort.CloseCOMPort();

end;

procedure TMain_F.pbCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TMain_F.EnableDisableControls(enable: boolean);
begin
  if not(enable) then
    begin
    ckAutoPoll.checked := FALSE;
    ttAutoPoll.enabled := FALSE;
    end;

  if not(enable) then
    begin
    ckAutoTXRXRand.checked := FALSE;
    ttAutoTXRXRand.enabled := FALSE;
    end;

  SDUEnableControl(pbClosePort, enable);
  SDUEnableControl(pbSendNow, enable);
  SDUEnableControl(edUserText, enable);
  SDUEnableControl(pbPoll, enable);
  SDUEnableControl(ckAutoPoll, enable);

  SDUEnableControl(ckAutoTXRXRand, enable);
  SDUEnableControl(pbTXRX, enable);
  SDUEnableControl(pbTXRXRandomPair, enable);
end;

procedure TMain_F.pbSetIntervalClick(Sender: TObject);
begin
  ttAutoPoll.interval := strtoint(edAutoInterval.text);
  ttAutoTXRXRand.interval :=  strtoint(edAutoInterval.text);
  
end;

procedure TMain_F.pbTXRXClick(Sender: TObject);
const
  TEST_STRING = 'Hello there!';
var
  i: integer;
begin
  ListBox1.Items.add('Writing out: '+TEST_STRING);
  if not(testPort.WriteData(TEST_STRING)) then
    begin
    ListBox1.Items.add('Error writing data out!');
    end
  else
    begin
    ListBox1.Items.add('Reading in data...');
    for i:=1 to length(TEST_STRING) do
      begin
      pbPollClick(nil);
      end;
    end;

end;

procedure TMain_F.pbTXRXRandomPairClick(Sender: TObject);
var
  testStringOut: string;
  testStringIn: string;
  addString: string;
begin
  randomize();
  testStringOut:= char(random(256));

  ListBox1.Items.add('Writing out: #'+inttostr(ord(testStringOut[1])));
  if not(testPort.WriteData(testStringOut)) then
    begin
    ListBox1.Items.add('Error writing data out!');
    end
  else
    begin
    addString := 'Reading in: Failed to read data from port';
    if testPort.ReadData(testStringIn, 1)<>-1 then
      begin
      if length(testStringIn)>0 then
        begin
        addString := 'Reading in: #'+inttostr(ord(testStringIn[1]));
        end
      else
        begin
        addString := 'Reading in: No data to read';
        end;
      end;

    if testStringIn=testStringout then
      begin
      addString := addString + ' (correct)';
      if rgTXRXPairBeep.itemindex = 2 then
        begin
        beep;
        end;
      end
    else
      begin
      addString := addString + ' -- ERROR!';
      if rgTXRXPairBeep.itemindex = 1 then
        begin
        beep;
        end;
      end;

    ListBox1.Items.add(addString);
  end;

end;

procedure TMain_F.ckAutoTXRXRandClick(Sender: TObject);
begin
  ttAutoTXRXRand.enabled := ckAutoTXRXRand.checked;

  edAutoInterval.enabled   := (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked));
  pbSetInterval.enabled    := (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked));
  lblInterval.enabled      := (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked));
  lblIntervalUnits.enabled := (not(ckAutoPoll.checked)) AND (not(ckAutoTXRXRand.checked));

end;

procedure TMain_F.pbSendNowClick(Sender: TObject);
begin
  if edUserText.text='' then
    begin
    ListBox1.Items.add('Silly - enter some text before trying to send it!');
    end
  else
    begin
    ListBox1.Items.add('Writing out: '+edUserText.text);
    if not(testPort.WriteData(edUserText.text)) then
      begin
      ListBox1.Items.add('Error writing data out!');
      end;
    end;

end;

END.

